/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef AUDIO_OUTPUT_H
#define AUDIO_OUTPUT_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <fx.h>

#ifdef HAVE_BS2B
#  include <bs2b.h>
#endif

#include "input.h"
#include "output.h"
#include "converter.h"

/*****************************************************************/

class audioThread;

/*****************************************************************/

enum state_t { STOP, PLAY, PAUSE };
enum msg_t { PRELOAD_SONG, SONG_ENDED, UPD_TIME };

class audio {

	friend class audioThread;

private:
	FXApp		*_app;
	int		_card;

	input		*_input;
	input		*_preload;
	output		*_output;

	state_t		_state;
	bool		_playing;

	audioThread	*_thread;

	int		_bufferSize;

	int 		_buffers;
	int 		_bufPerSec;
	int		_seconds;

	FXGUISignal	*_guiMsg;

	int		_volume;

	converter	*_converter;
#ifdef HAVE_BS2B
	t_bs2bdp	_bs2bdp;
#endif
private:
	audio() {};
	audio(const audio&);
	audio& operator=(const audio&);

	///
	void loop();

public:
	audio(FXApp *app, FXObject *o, FXSelector guiMsg);
	~audio();

	/// Start stream
	bool play(input* i);

	/// Pause stream
	void pause();

	/// Stop stream
	bool stop();

	/// Get state
	state_t state() const { return _state; }

	/// Get seconds
	FXint seconds() const { return _seconds; }

	/// Open config dialog
	void config(FXWindow* win);

	/// Set volume
	void volume(const int vol);

	/// Get volume
	int volume() { return _volume; }

	/// Check if gapless playback is supported
	bool gapless(input* const i);
};

/*****************************************************************/

#include "configWindow.h"

class audioConfig : public configWindow {
	FXDECLARE(audioConfig)
private:
	FXListBox	*_apiList;
	FXListBox	*_cardList;

private:
	audioConfig() {};
	audioConfig(const audioConfig&);
	audioConfig& operator=(const audioConfig&);

	void setCards(const output* const audio);

public:
	audioConfig(FXWindow* win);
	virtual ~audioConfig() {};

public:
	enum{
		ID_API=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdApi(FXObject*, FXSelector, void*);

public:
	void api(const char* apiName);

	int api() const { return _apiList->getCurrentItem(); }

	void card(const int card);

	int card() const { return _cardList->getCurrentItem(); }
};

/*****************************************************************/

class audioThread : public FXThread {
private:
	audio	*_audio;

protected:
	FXint run();

public:
	audioThread(audio* a)
		: FXThread(), _audio(a) {};
	~audioThread() {};
};

#endif
