/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "configWindow.h"
#include "iconFactory.h"
#include "LinkLabel.h"

FXIMPLEMENT(configWindow, FXDialogBox, 0, 0)

configWindow::configWindow(FXWindow* win, const char* title, const char* credits, const char* link) :
	FXDialogBox(win, FXString::null, DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0) {

	setTitle(FXStringFormat("%s %s", title, tr("Config")));

	FXVerticalFrame* main=new FXVerticalFrame(this, 0, 0,0,0,0, 0,0,0,0, 0,0);

	if (credits) {
		FXLabel* cred=new FXLabel(main, credits, 0, LAYOUT_FILL_X);
		cred->setBackColor(FXRGB(255,255,255));
	}

	if (link) {
		FXLinkLabel* lnk=new FXLinkLabel(main, link, 0, LAYOUT_FILL_X);
		lnk->setBackColor(FXRGB(255,255,255));
	}

	new FXSeparator(main);

	FXHorizontalFrame* top=new FXHorizontalFrame(main, LAYOUT_FILL_X);

	_matrix=new FXMatrix(top, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL);
	_extraLeft=new FXHorizontalFrame(top, LAYOUT_FILL);

	_extraBottom=new FXVerticalFrame(main, LAYOUT_FILL);

	new FXSeparator(this);

	FXHorizontalFrame* bottom=new FXHorizontalFrame(this, LAYOUT_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
	FXButton* initial=new FXButton(bottom, tr("&OK"), GET_ICON(getApp(), icon_dialogok),
		this, ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X);
	new FXButton(bottom, tr("&Cancel"), GET_ICON(getApp(), icon_dialogcancel),
		this, ID_CANCEL, BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X);
	initial->setFocus();
}
