/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "converter.h"

converter::converter(int srIn, int srOut, unsigned int size, unsigned int channels, unsigned int prec) :
	_channels(channels),
	_prec(prec),
	_int(0),
	_fract(0),
	_data(0) {

	_sampleSize=_prec*_channels;

	_iRate=(FXuchar)(srIn/srOut);
	_fRate=(FXuint)((((FXulong)(srIn-srOut*_iRate))<<32)/srOut);
	FXTRACE((10, "Conversion ratio %f\n", (float)srIn/srOut));

	const FXuint tmp=(size/_sampleSize)*srIn;
	_bufsize=tmp/srOut;
	if ((_bufsize*srOut)!=tmp)
		_bufsize++;

	_bufsize*=_sampleSize;
	_buffer=new char[_bufsize];
}

converter::~converter() {

	delete [] _buffer;
}

int converter::convert(char* out, unsigned int len) {

	switch (_prec) {
	case 1:
		return _convert<FXuchar>((FXuchar*)out, len);
	case 2:
		return _convert<FXshort>((FXshort*)out, len);
	default:
		return 0;
	}
}

template <class T>
int converter::_convert(T* buf, unsigned int len) {

	T* const in=(T*)_buffer;
	T* out=buf;
	const T* const end=buf+len;

	if (!_data)
		len--;
	do {
		FXulong tmp=(FXulong)_fract;
		unsigned int idx=_int*_channels;
		for (unsigned int c=0; c<_channels; c++,idx++) {
			const T val=in[idx];
			*out++=val+(T)(((FXulong)(in[idx+_channels]-val)*tmp)>>32);
		}
		tmp+=(FXulong)_fRate;
		_int+=_iRate+(FXushort)(tmp>>32);
		_fract=(FXuint)tmp;
	} while ((out<end) && (_int<len));

	_int-=len;
	if (!_int) {
		for (unsigned int c=0; c<_channels; c++)
			in[c]=in[len*_channels+c];
		_data=_sampleSize;
	} else
		_data=0;

	return (int)(out-buf)*_prec;
}
