/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef FFMPEG_H
#define FFMPEG_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

extern "C" {
#ifdef HAVE_FFMPEG_AVFORMAT_H
#  include <ffmpeg/avformat.h>
#  define HAVE_FFMPEG
#elif defined HAVE_LIBAVFORMAT_AVFORMAT_H
#  include <libavformat/avformat.h>
#  define HAVE_FFMPEG
#endif
}

#ifdef HAVE_FFMPEG

#include "inputPlugin.h"
#include "AutoDLL.h"

#include <fx.h>

/*****************************************************************/

class ffmpeg : public inputPlugin {

private:
	AVStream	*_audioStream;
	AVFormatContext	*_formatContext;
	int		_audioStreamIndex;
	FXuint		_decodeBufOffset;
	FXuchar		_decodeBuf[AVCODEC_MAX_AUDIO_FRAME_SIZE*2];
	AVPacket	_packet;
	FXuint		_packetOffset;

	static const AutoDLL	_avformat;
	static const AutoDLL	_avcodec;

	static FXString		_ext;

private:
	static int (*dl_av_open_input_file)(AVFormatContext**, const char*, AVInputFormat*, int, AVFormatParameters*);
	static void (*dl_av_close_input_file)(AVFormatContext*);
	static int (*dl_av_find_stream_info)(AVFormatContext*);
	static int (*dl_av_read_frame)(AVFormatContext*, AVPacket*);
	static int (*dl_av_seek_frame)(AVFormatContext*, int, int64_t, int);

	static AVCodec* (*dl_avcodec_find_decoder)(enum CodecID);
	static int (*dl_avcodec_open)(AVCodecContext*, AVCodec*);
	static int (*dl_avcodec_decode_audio)(AVCodecContext*, int16_t*, int*, uint8_t*, int);
	static void (*dl_avcodec_flush_buffers)(AVCodecContext*);
	static int (*dl_avcodec_close)(AVCodecContext*);

private:
	/// Open selected stream
	bool openStream(AVFormatContext* fc, const int streamIndex);

public:
	ffmpeg(FXApp* a);
	~ffmpeg();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory(FXApp* a) { return new ffmpeg(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const {
		return _audioStream?
			_audioStream->codec->sample_rate:
			0;
	}

	/// Get channels
	const FXint channels() const {
		return _audioStream?
			_audioStream->codec->channels:
			0;
	}

	/// Get precision
	const FXint precision() const { return 2; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);
};

/*****************************************************************/

#include "configWindow.h"

class ffmpegConfig : public configWindow {
	FXDECLARE(ffmpegConfig)

private:
	ffmpegConfig() {};
	ffmpegConfig(const ffmpegConfig&);
	ffmpegConfig& operator=(const ffmpegConfig&);

public:
	ffmpegConfig(FXWindow* win);
	virtual ~ffmpegConfig() {};
};

#endif

#endif
