/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GAME_H
#define GAME_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_GME
#include "gme/Music_Emu.h"

#include <fx.h>
#include "inputPlugin.h"
#include "stil.h"

/*****************************************************************/

typedef struct {
	FXint		samplerate;
	FXString	asmaPath;
} gmeConfig_t;

/*****************************************************************/

class game : public inputPlugin {

private:
	Music_Emu	*_emu;
	gmeConfig_t	_settings;
	bool		_hasStilInfo;

	STIL		*_stil;

private:
	void openAsma(const FXString& asmaPath);

	void getInfo();

	bool check(const char* error);

public:
	game(FXApp* a);
	~game();

	static const char name[];

	/// Factory method
	static input* factory(FXApp* a) { return new game(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get number of subtunes
	FXuint subtunes() const { return _emu?_emu->track_count():0; }

	/// Get current subtune
	FXuint subtune() const { return _emu?_emu->current_track()+1:0; }

	/// Change subtune
	bool subtune(const FXuint i);

	/// Get samplerate
	const FXint samplerate() const { return _settings.samplerate; }

	/// Get channels
	const FXint channels() const { return 2; }

	/// Get precision
	const FXint precision() const { return 2; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);
};

/*****************************************************************/

#include "configWindow.h"

class gameConfig : public configWindow {
	FXDECLARE(gameConfig)

private:
	gmeConfig_t	_settings;
	FXDataTarget	_asmaDir;

private:
	gameConfig() {};
	gameConfig(const gameConfig&);
	gameConfig& operator=(const gameConfig&);

public:
	gameConfig(FXWindow* win, gmeConfig_t settings);
	virtual ~gameConfig() {};

	/// Get settings
	gmeConfig_t settings() const { return _settings; }

public:
	enum{
		ID_SAMPLERATE=FXDialogBox::ID_LAST,
		ID_ASMA,
		ID_LAST,
	};

public:
	long onCmdSamplerate(FXObject*, FXSelector, void*);
	long onCmdAsma(FXObject*, FXSelector, void*);
};

#endif

#endif
