/*
 *  Copyright (C) 2007-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef HVL_H
#define HVL_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_HVL
extern "C" {
#include "hvl_replay/hvl_replay.h"
}
#include <fx.h>
#include "inputPlugin.h"

/*****************************************************************/

typedef struct {
	FXint		samplerate;
} hvlConfig_t;

/*****************************************************************/

class hvl : public inputPlugin {

private:
	struct hvl_tune *_tune;
	hvlConfig_t	_settings;

	int		_subtune;

	int		_left;
	FXuint		_size;
	char		*_buffer;

public:
	hvl(FXApp *a);
	~hvl();

	static const char name[];

	/// Factory method
	static input* factory(FXApp* a) { return new hvl(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get number of subtunes
	FXuint subtunes() const { return _tune?_tune->ht_SubsongNr:0; }

	/// Get current subtune
	FXuint subtune() const { return _tune?_tune->ht_SongNum:0; }

	/// Change subtune
	bool subtune(const FXuint i);

	/// Get samplerate
	const FXint samplerate() const { return _settings.samplerate; }

	/// Get channels
	const FXint channels() const { return 2; }

	/// Get precision
	const FXint precision() const { return 2; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);
};

/*****************************************************************/

#include "configWindow.h"

class hvlConfig : public configWindow {
	FXDECLARE(hvlConfig)

private:
	hvlConfig_t	_settings;

private:
	hvlConfig() {};
	hvlConfig(const hvlConfig&);
	hvlConfig& operator=(const hvlConfig&);

public:
	hvlConfig(FXWindow* win, hvlConfig_t settings);
	virtual ~hvlConfig() {};

	/// Get settings
	hvlConfig_t settings() const { return _settings; }

public:
	enum{
		ID_SAMPLERATE=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdSamplerate(FXObject*, FXSelector, void*);
};

#endif

#endif
