/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef INPUT_H
#define INPUT_H

#include <fx.h>

// MPRIS Metadata
// http://wiki.xmms2.xmms.se/index.php/MPRIS_Metadata
#define TITLE		"title"
#define ARTIST		"artist"
#define ALBUM		"album"
#define TRACK		"tracknumber"
#define TIME		"time"
#define MTIME		"mtime"
#define GENRE		"genre"
#define COMMENT		"comment"
#define RATING		"rating"
#define YEAR		"year"
#define ARTURL		"arturl"

//#define BITRATE		"audio-bitrate"
//#define SAMPLERATE	"audio-samplerate"

/**
 * interface class for input backends
*/
class input {
public:
	virtual ~input() {};

	/// Get supported extensions
	virtual FXString ext() const =0;

	/// Filter dir content
	virtual FXint filter(FXString* files, FXint items) =0;

	/// Get song info
	virtual FXint moreInfo(FXint i) const =0;
	virtual const char* getKey(FXuint num) const =0;
	virtual const char* getInfo(FXuint num) const =0;
	virtual const char* getInfo(const char* info) const =0;

	/// Get song duration
	virtual const FXint time() const =0;

	/// Get samplerate
	virtual const FXint samplerate() const =0;

	/// Get channels
	virtual const FXint channels() const =0;

	/// Get precision
	virtual const FXint precision() const =0;

	/// Get filetype icon
	virtual FXIcon* icon() const =0;

	/// Open file
	virtual bool open(const FXString& fileName) =0;

	/// Close file
	virtual void close() =0;

	/// Rewind to start
	virtual const bool rewind() =0;

	/// Open config dialog
	virtual const bool config(FXWindow* win) =0;

	/// Callback function
	virtual int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds) =0;

	/// Song is loaded
	virtual const bool songLoaded() const =0;

	/// Get number of subtunes
	virtual FXuint subtunes() const =0;

	/// Get current subtune
	virtual FXuint subtune() const =0;

	/// Change subtune
	virtual bool subtune(const FXuint i) =0;

	/// Gapless support
	virtual const bool gapless() =0;
};

#endif
