/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef INPUT_PLUGIN_H
#define INPUT_PLUGIN_H

#include "input.h"

#define SECTION	FXStringFormat("%s Settings", _name).text()

/*****************************************************************/

class inputPlugin : public input {

private:
	FXApp		*_app;
	const char	*_name;
	FXIcon		*_icon;

	FXint		_time;

	bool		_songLoaded;

	FXStringDict	_infos;

private:
	inputPlugin();
	inputPlugin(const inputPlugin&);
	inputPlugin& operator=(const inputPlugin&);

protected:
	inputPlugin(FXApp* app, const char name[], const unsigned char iconType[]=0);

	/// Set song duration
	void time(FXint newTime) { _time=newTime; }

	/// Load int setting
	FXint load(const FXchar* key, FXint defVal) {
		return _app->reg().readIntEntry(SECTION, key, defVal);
	}

	/// Load string setting
	FXString load(const FXchar* key, FXString defVal) {
		return FXString(_app->reg().readStringEntry(SECTION, key, defVal.text()));
	}

	/// Save int setting
	void save(const FXchar* key, FXint value) {
		_app->reg().writeIntEntry(SECTION, key, value);
	}

	/// Save string setting
	void save(const FXchar* key, FXString value) {
		_app->reg().writeStringEntry(SECTION, key, value.text());
	}

	/// Song is loaded
	void songLoaded(const bool isLoaded) { _songLoaded=isLoaded; if (!isLoaded) _infos.clear(); }

	/// Append song info
	void addInfo(const char* type, const char* info) { if (info && info[0]) _infos.insert(type, info); }
	void addInfo(const char* type, int info) { if (info) _infos.insert(type, FXStringVal(info).text()); }

	/// Remove song info
	void delInfo(const char* type) { _infos.remove(type); }

	/// Find song info
	const FXchar* findInfo(const char* type) const { return _infos.find(type); }

public:
	virtual ~inputPlugin();

	/// Filter dir content
	virtual FXint filter(FXString* files, FXint items);

	/// Get backend name
	const char* name() const { return _name; }

	/// Get song info
	virtual FXint moreInfo(FXint i) const {
		int j=(i<0)?_infos.first():_infos.next(i);
		return (j<_infos.size())?j:-1;
	}
	virtual const char* getKey(FXuint num) const { return _infos.key(num); }
	virtual const char* getInfo(FXuint num) const { return _infos.data(num); }
	virtual const char* getInfo(const char* info) const { return _infos.find(info); }

	/// Get song duration
	const FXint time() const { return _time; }

	/// Get filetype icon
	FXIcon* icon() const { return _icon; }

	/// Song is loaded
	const bool songLoaded() const { return _songLoaded; }

	/// Get number of subtunes
	virtual FXuint subtunes() const { return 0; }

	/// Get current subtune
	virtual FXuint subtune() const { return 0; }

	/// Change subtune
	virtual bool subtune(const FXuint i) { return false; }

	/// Gapless support
	virtual const bool gapless() { return false; };
};

#endif
