/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "mp3.h"

#ifdef HAVE_MAD_H

#include "utils.h"
#include "tag.h"
#include "loadsym.h"

// mp3|mp2
#define EXT	"*.mp3"

#define CREDITS		FXStringFormat("%s\n%s", mp3::dl_mad_version, mp3::dl_mad_copyright).text()
#define LINK		"http://www.underbit.com/products/mad/"

const char mp3::name[]="Mad";

#ifdef _WIN32
# define MADLIB	"libmad.dll"
#else
# define MADLIB	"libmad.so"
#endif

const AutoDLL mp3::_dll(MADLIB);

const char* mp3::dl_mad_version=0;
const char* mp3::dl_mad_copyright=0;

void (*mp3::dl_mad_stream_init)(struct mad_stream*)=0;
void (*mp3::dl_mad_stream_finish)(struct mad_stream*)=0;
void (*mp3::dl_mad_stream_buffer)(struct mad_stream*, unsigned char const*, unsigned long)=0;
int (*mp3::dl_mad_stream_sync)(struct mad_stream*)=0;

void (*mp3::dl_mad_header_init)(struct mad_header*)=0;
int (*mp3::dl_mad_header_decode)(struct mad_header*, struct mad_stream*)=0;

void (*mp3::dl_mad_frame_init)(struct mad_frame*)=0;
void (*mp3::dl_mad_frame_finish)(struct mad_frame*)=0;
int (*mp3::dl_mad_frame_decode)(struct mad_frame*, struct mad_stream*)=0;
void (*mp3::dl_mad_frame_mute)(struct mad_frame*)=0;

void (*mp3::dl_mad_synth_init)(struct mad_synth*)=0;
void (*mp3::dl_mad_synth_mute)(struct mad_synth*)=0;
void (*mp3::dl_mad_synth_frame)(struct mad_synth*, struct mad_frame const*)=0;

/*****************************************************************/

#if defined(HAVE_MMAP) || defined(_WIN32)
int mp3::fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds) {

	unsigned int n=0;
	const unsigned int size=bufferSize/_settings.precision;
	FXshort* buf=(FXshort*)buffer;
	do {
		if (!_idx) {
			int res;
			do {
				res=dl_mad_frame_decode(&_frame, &_stream);
				if (res && !MAD_RECOVERABLE(_stream.error))
					return (n*_settings.precision);
			} while (res);

			dl_mad_synth_frame(&_synth, &_frame);
		}

		for (unsigned int j=0; j<_channels; j++) {
			if (_settings.precision==2)
				buf[n++]=_quant->get16((void*)&_synth.pcm.samples[j][_idx], j);
			else
				buffer[n++]=_quant->get8((void*)&_synth.pcm.samples[j][_idx], j);
		}

		if (++_idx>=_synth.pcm.length)
			_idx=0;
	} while (n<size);

	return (n*_settings.precision);
}
#else
//TODO
#endif

/*****************************************************************/

bool mp3::init() {

	if (!_dll.loaded())
		return false;

	LOADSYM(_dll, mad_version, const char*)
	LOADSYM(_dll, mad_copyright, const char*)
	LOADSYM(_dll, mad_stream_init, void(*)(mad_stream*))
	LOADSYM(_dll, mad_stream_finish, void(*)(mad_stream*))
	LOADSYM(_dll, mad_stream_buffer, void(*)(mad_stream*, unsigned char const*, unsigned long))
	LOADSYM(_dll, mad_stream_sync, int(*)(mad_stream*))
	LOADSYM(_dll, mad_header_init, void(*)(mad_header*))
	LOADSYM(_dll, mad_header_decode, int(*)(mad_header*, mad_stream*))
	LOADSYM(_dll, mad_frame_init, void(*)(mad_frame*))
	LOADSYM(_dll, mad_frame_finish, void(*)(mad_frame*))
	LOADSYM(_dll, mad_frame_decode, int(*)(mad_frame*, mad_stream*))
	LOADSYM(_dll, mad_frame_mute, void(*)(mad_frame*))
	LOADSYM(_dll, mad_synth_init, void(*)(mad_synth*))
	LOADSYM(_dll, mad_synth_mute, void(*)(mad_synth*))
	LOADSYM(_dll, mad_synth_frame, void(*)(mad_synth*, mad_frame const*))

	return true;
}

bool mp3::supports(const char* fileName) {

	return FXPath::match(FXString(EXT), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
}

inline FXString mp3::ext() const { return FXString(EXT); }

mp3::mp3(FXApp* a) :
	inputPlugin(a, name),
	_quant(0) {

	_settings.precision=(load("Bits", 16))>>3;
	_settings.fastscan=load("Fastscan", FALSE);
}

mp3::~mp3() {

	close();

	save("Bits", (_settings.precision<<3));
	save("Fastscan", _settings.fastscan);
}

#if defined(HAVE_MMAP) || defined(_WIN32)
bool mp3::open(const FXString& fileName) {

	close();

	if (!_memMap.mapFile(fileName))
		return false;

	const tag *tagPtr=new tag(&_memMap);
	FXTRACE((10, "offsBegin: %d.\n", tagPtr->offsBegin()));
	FXTRACE((10, "offsEnd: %d.\n", tagPtr->offsEnd()));

	_data=(FXuchar*)_memMap.base()+tagPtr->offsBegin();
	_length=_memMap.length()-(tagPtr->offsBegin()+tagPtr->offsEnd());

	dl_mad_stream_init(&_stream);
	mad_stream_options(&_stream, MAD_OPTION_IGNORECRC);

	int res;

	dl_mad_stream_buffer(&_stream, _data, _length);
	if (dl_mad_stream_sync(&_stream)<0)
		goto error;

	mad_header header;
	dl_mad_header_init(&header);
	do {
		res=dl_mad_header_decode(&header, &_stream);
	} while (res);

	{
	const unsigned int offset=((header.flags&MAD_FLAG_LSF_EXT)?(header.mode?21:13):(header.mode?36:21));
	const int xingFrames=getXing((char*)_stream.buffer+offset);

	if (xingFrames) {
		do {
			res=dl_mad_header_decode(&header, &_stream);
		} while (res);
	}

	_samplerate=header.samplerate;
	if (!_samplerate)
		goto error;

	unsigned int samples=32*MAD_NSBSAMPLES(&header);
	_channels=MAD_NCHANNELS(&header);

	if (xingFrames)
		time(xingFrames*samples/_samplerate);
	else {
		bool cbr=true;
		int count=0;
		const unsigned int bitrate=header.bitrate;
		do {
			res=dl_mad_header_decode(&header, &_stream);
			if (!res) {
				samples+=32*MAD_NSBSAMPLES(&header);
				if (_samplerate!=header.samplerate) {
					fxmessage("Samplerate change not supported\n");
					mad_header_finish(&header);
					goto error;
				}
				if (cbr && (bitrate!=header.bitrate)) {
					FXTRACE((10, "VBR\n"));
					cbr=false;
				}
				if ((++count>10) && (cbr || _settings.fastscan)) {
					time(_length/(bitrate>>3));
					break;
				}
			}
		} while ((!res) || MAD_RECOVERABLE(_stream.error));

		if ((!cbr) && (!_settings.fastscan))
			time(samples/_samplerate);
	}
	}

	mad_header_finish(&header);

	dl_mad_stream_buffer(&_stream, _data, _length);
	dl_mad_stream_sync(&_stream);

	dl_mad_frame_init(&_frame);
	dl_mad_synth_init(&_synth);

	addInfo(TITLE, tagPtr->title());
	addInfo(ARTIST, tagPtr->artist());
	addInfo(ALBUM, tagPtr->album());
	addInfo(GENRE, tagPtr->genre());
	addInfo(YEAR, tagPtr->year());
	addInfo("publisher", tagPtr->publisher());
	addInfo(TRACK, tagPtr->track());
	addInfo(COMMENT, tagPtr->comment());

	delete tagPtr;

	_idx=0;

	_quant=new requantizeInt(_settings.precision, MAD_F_FRACBITS);

	songLoaded(true);
	return true;

error:
	dl_mad_stream_finish(&_stream);
	delete tagPtr;
	_memMap.unmap();

	return false;
}

int mp3::getXing(const char* buf) {

	if ((!tag::isFrame(buf, "Xing")) &&
	(!tag::isFrame(buf, "Info")))
		return 0;

	FXTRACE((10, "Xing tag found.\n"));

	if (!buf[7]&0x01)
		// No Frames field
		return 0;

	const int frames=tag::getFrameSize(buf+8, false);
	FXTRACE((10, "Frames: %d.\n", frames));

	return frames;
}

void mp3::close() {

	if (!songLoaded())
		return;

	dl_mad_frame_finish(&_frame);
	mad_synth_finish(&_synth);
	dl_mad_stream_finish(&_stream);

	_memMap.unmap();

	delPtr(_quant);

	songLoaded(false);
}

const bool mp3::rewind() {

	if (songLoaded()) {
		dl_mad_stream_buffer(&_stream, _data, _length);
		dl_mad_stream_sync(&_stream);

		dl_mad_frame_mute(&_frame);
		dl_mad_synth_mute(&_synth);

		_idx=0;

		return true;
	}
	return false;
}
#else
//TODO
#endif

const bool mp3::config(FXWindow* win) {

	mp3Config cWin(win, _settings);
	if (cWin.execute(PLACEMENT_OWNER)) {
		_settings=cWin.settings();
		return true;
	}
	return false;
}

/*****************************************************************/

FXDEFMAP(mp3Config) mp3ConfigMap[] = {
	// Message_Type		ID				Message_Handler
	FXMAPFUNC(SEL_COMMAND,	mp3Config::ID_BITS,		mp3Config::onCmdBits),
};

FXIMPLEMENT(mp3Config, configWindow, mp3ConfigMap, ARRAYNUMBER(mp3ConfigMap))

mp3Config::mp3Config(FXWindow* win, mp3Config_t settings) :
	configWindow(win, mp3::name, CREDITS, LINK),
	_settings(settings) {

	FXint val;

	_scan.connect(_settings.fastscan);

	new FXLabel(matrix(), tr("Bits"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *bitBox=new FXListBox(matrix(), this, mp3Config::ID_BITS, LAYOUT_FILL|FRAME_NORMAL);
	bitBox->fillItems("8\n16");
	bitBox->setNumVisible(2);
	switch (_settings.precision) {
	case 1:
		val=0;
		break;
	default:
	case 2:
		val=1;
		break;
	}
	bitBox->setCurrentItem(val);

	new FXVerticalSeparator(extraLeft());

	FXVerticalFrame *vert=new FXVerticalFrame(extraLeft());
	new FXCheckButton(vert, tr("Fast scan\tScan only few frame of file, time detection may be inaccurate"), &_scan, FXDataTarget::ID_VALUE);
}

long mp3Config::onCmdBits(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		_settings.precision=1;
		break;
	case 1:
		_settings.precision=2;
		break;
	}
	return 1;
}

#endif
