/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef MP3_H
#define MP3_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_MAD_H
#include <mad.h>

#include "inputPlugin.h"
#include "requantize.h"
#include "AutoDLL.h"

#include <fx.h>

#if defined(HAVE_MMAP) || defined(_WIN32)
#  include <FXMemMap.h>
#else
#  error
#endif

/*****************************************************************/

typedef struct {
	FXbool		fastscan;
	FXint		precision;
} mp3Config_t;

/*****************************************************************/

class mp3 : public inputPlugin {

private:
	mad_stream	_stream;
	mad_frame	_frame;
	mad_synth	_synth;
	FXuint		_idx;

	mp3Config_t	_settings;

	FXuint		_samplerate;
	FXuint		_channels;
	FXuint		_scalebits;

	FXuchar		*_data;
	FXuint		_length;

	requantize	*_quant;

#if defined(HAVE_MMAP) || defined(_WIN32)
	FXMemMap	_memMap;
#else
	FXFile		_file;
#endif

	static const AutoDLL	_dll;

public:
	static const char* dl_mad_version;
	static const char* dl_mad_copyright;

private:
	static void (*dl_mad_stream_init)(struct mad_stream*);
	static void (*dl_mad_stream_finish)(struct mad_stream*);
	static void (*dl_mad_stream_buffer)(struct mad_stream*, unsigned char const *, unsigned long);
	static int (*dl_mad_stream_sync)(struct mad_stream*);

	static void (*dl_mad_header_init)(struct mad_header*);
	static int (*dl_mad_header_decode)(struct mad_header*, struct mad_stream*);

	static void (*dl_mad_frame_init)(struct mad_frame*);
	static void (*dl_mad_frame_finish)(struct mad_frame*);
	static int (*dl_mad_frame_decode)(struct mad_frame*, struct mad_stream*);
	static void (*dl_mad_frame_mute)(struct mad_frame*);

	static void (*dl_mad_synth_init)(struct mad_synth*);
	static void (*dl_mad_synth_mute)(struct mad_synth*);
	static void (*dl_mad_synth_frame)(struct mad_synth*, struct mad_frame const*);

private:
	int getXing(const char* buf);

public:
	mp3(FXApp* a);
	~mp3();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory(FXApp *a) { return new mp3(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const { return _samplerate; }

	/// Get channels
	const FXint channels() const { return _channels; }

	/// Get precision
	const FXint precision() const { return _settings.precision; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);
};

/*****************************************************************/

#include "configWindow.h"

class mp3Config : public configWindow {
	FXDECLARE(mp3Config)

private:
	mp3Config_t	_settings;

	FXDataTarget	_scan;

private:
	mp3Config() {};
	mp3Config(const mp3Config&);
	mp3Config& operator=(const mp3Config&);

public:
	mp3Config(FXWindow* win, mp3Config_t settings);
	virtual ~mp3Config() {};

	/// Get settings
	mp3Config_t settings() const { return _settings; }

public:
	enum{
		ID_BITS=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdBits(FXObject*, FXSelector, void*);
};

#endif

#endif
