/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef MPC_H
#define MPC_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_LIBMPCDEC

#ifdef HAVE_MPC_MPCDEC_H
#  include <mpc/mpcdec.h>
#  define DATAPARM	mpc_reader *p_reader
#  define DATAFILE	(p_reader->data)
#  define SV8
#else
#  include <mpcdec/mpcdec.h>
#  define DATAPARM	void *data
#  define DATAFILE	data
#endif

#include "inputPlugin.h"
#include "requantize.h"

#include <fx.h>

/*****************************************************************/

typedef struct {
	FXint		precision;
} mpcConfig_t;

/*****************************************************************/

class mpc : public inputPlugin {
private:
#ifdef SV8
	mpc_demux* 	_demux;
#else
	mpc_decoder	_decoder;
#endif
	mpc_streaminfo	_si;
	mpc_reader	_mpcReader;

	mpcConfig_t	_settings;

	MPC_SAMPLE_FORMAT	_buffer[MPC_DECODER_BUFFER_LENGTH];
	FXuint			_bufIndex;
	FXuint			_bufLen;

	requantize	*_quant;

	FXFile		_file;

private:
	static mpc_int32_t	read_func(DATAPARM, void *ptr, mpc_int32_t size);
	static mpc_bool_t	seek_func(DATAPARM, mpc_int32_t offset);
	static mpc_int32_t	tell_func(DATAPARM);
	static mpc_int32_t	get_size_func(DATAPARM);
	static mpc_bool_t	canseek_func(DATAPARM);

public:
	mpc(FXApp *a);
	~mpc();

	static const char name[];

	/// Factory method
	static input* factory(FXApp* a) { return new mpc(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const { return _si.sample_freq; }

	/// Get channels
	const FXint channels() const { return _si.channels; }

	/// Get precision
	const FXint precision() const { return _settings.precision; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return songLoaded()?_si.is_true_gapless:true; };
};

/*****************************************************************/

#include "configWindow.h"

class mpcConfig : public configWindow {
	FXDECLARE(mpcConfig)

private:
	mpcConfig_t	_settings;

private:
	mpcConfig() {};
	mpcConfig(const mpcConfig&);
	mpcConfig& operator=(const mpcConfig&);

public:
	mpcConfig(FXWindow* win, mpcConfig_t settings);
	virtual ~mpcConfig() {};

	/// Get settings
	mpcConfig_t settings() const { return _settings; }
public:
	enum{
		ID_BITS=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdBits(FXObject*, FXSelector, void*);
};

#endif

#endif
