/*
 *  Copyright (C) 2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef MPEG_H
#define MPEG_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_MPG123_H
#include <mpg123.h>

#include "inputPlugin.h"
#include "AutoDLL.h"

#include <fx.h>
#include <new>
#include <FXArray.h>

/*****************************************************************/

typedef struct {
	FXbool		fastscan;
	FXString	decoder;
} mpegConfig_t;

/*****************************************************************/

class mpeg : public inputPlugin {

private:
	mpg123_handle	*_handle;

	mpegConfig_t	_settings;

	long		_samplerate;
	int		_channels;

	int		_fd;

	static FXFile	_file[2];
	static int	_count;

	static int		_status;
	static const AutoDLL	_dll;

	static ssize_t	read_func(int, void*, size_t);
	static off_t	seek_func(int, off_t, int);

public:
	static FXString		_decoders;

private:
	static int (*dl_mpg123_init)(void);
	static void (*dl_mpg123_exit)(void);
	static mpg123_handle* (*dl_mpg123_new)(const char*, int*);
	static void (*dl_mpg123_delete)(mpg123_handle*);
	static const char* (*dl_mpg123_plain_strerror)(int);
	static int (*dl_mpg123_open_fd)(mpg123_handle*, int);
	static int (*dl_mpg123_close)(mpg123_handle*);
	static int (*dl_mpg123_scan)(mpg123_handle*);
	static int (*dl_mpg123_getformat)(mpg123_handle*, long*, int*, int*);
	static off_t (*dl_mpg123_length)(mpg123_handle*);
	static int (*dl_mpg123_read)(mpg123_handle*, unsigned char*, size_t, size_t*);
	static off_t (*dl_mpg123_seek)(mpg123_handle*, off_t, int);
	static int (*dl_mpg123_id3)(mpg123_handle*, mpg123_id3v1**, mpg123_id3v2**);
	static const char** (*dl_mpg123_supported_decoders)(void);
	static int (*dl_mpg123_replace_reader)(mpg123_handle*, ssize_t(*)(int, void *, size_t), off_t(*)(int, off_t, int));

public:
	mpeg(FXApp* a);
	~mpeg();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory(FXApp* a) { return new mpeg(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const { return _samplerate; }

	/// Get channels
	const FXint channels() const { return _channels; }

	/// Get precision
	const FXint precision() const { return 2; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; };
};

/*****************************************************************/

#include "configWindow.h"

class mpegConfig : public configWindow {
	FXDECLARE(mpegConfig)

private:
	mpegConfig_t	_settings;

	FXDataTarget	_fastscan;

private:
	mpegConfig() {};
	mpegConfig(const mpegConfig&);
	mpegConfig& operator=(const mpegConfig&);

public:
	mpegConfig(FXWindow* win, mpegConfig_t settings);
	virtual ~mpegConfig() {};

	/// Get settings
	mpegConfig_t settings() const { return _settings; }

public:
	enum{
		ID_DECODER=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdDecoder(FXObject*, FXSelector, void*);

};

#endif

#endif
