/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef OGG_H
#define OGG_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_VORBIS

#include "inputPlugin.h"

#include <fx.h>
#include <vorbis/vorbisfile.h>

/*****************************************************************/

typedef struct {
	FXint		precision;
} oggConfig_t;

/*****************************************************************/

class ogg : public inputPlugin {
private:
	OggVorbis_File	*_vf;
	vorbis_info	*_vi;

	oggConfig_t	_settings;

	FXFile		_file;

private:
	bool getComment(const char* orig, FXString* dest, const char* type);

	static ov_callbacks	vorbis_callbacks;

	static size_t	read_func(void *ptr, size_t size, size_t nmemb, void *datasource);
	static int	seek_func(void *datasource, ogg_int64_t offset, int whence);
	static int	close_func(void *datasource);
	static long	tell_func(void *datasource);

public:
	ogg(FXApp *a);
	~ogg();

	static const char name[];

	/// Factory method
	static input* factory(FXApp* a) { return new ogg(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const { return _vi?_vi->rate:0; }

	/// Get channels
	const FXint channels() const { return _vi?_vi->channels:0; }

	/// Get precision
	const FXint precision() const { return _vi?_settings.precision:0; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; };
};

/*****************************************************************/

#include "configWindow.h"

class oggConfig : public configWindow {
	FXDECLARE(oggConfig)

private:
	oggConfig_t _settings;

private:
	oggConfig() {};
	oggConfig(const oggConfig&);
	oggConfig& operator=(const oggConfig&);

public:
	oggConfig(FXWindow* win, oggConfig_t settings);
	virtual ~oggConfig() {};

	/// Get settings
	oggConfig_t settings() const { return _settings; }
public:
	enum{
		ID_BITS=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdBits(FXObject*, FXSelector, void*);
};

#endif

#endif
