/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "sid.h"

#ifdef HAVE_SIDPLAY2

#ifdef HAVE_SIDPLAY_BUILDERS_RESID_H
# include <sidplay/builders/resid.h>
#endif

#ifdef HAVE_SIDPLAY_BUILDERS_HARDSID_H
# include <sidplay/builders/hardsid.h>
#endif

#include <FX88591Codec.h>
#include "utils.h"

// created by reswrap from file sid.gif
extern const unsigned char iconSid[126]={
  0x47,0x49,0x46,0x38,0x39,0x61,0x10,0x00,0x10,0x00,0xf2,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x86,0x00,0x00,0xff,0x86,0x00,0x00,0xff,0x00,0x00,0x86,0x82,0x86,0xcc,
  0xcc,0xcc,0x00,0x00,0x00,0x21,0xf9,0x04,0x01,0x00,0x00,0x06,0x00,0x2c,0x00,0x00,
  0x00,0x00,0x10,0x00,0x10,0x00,0x00,0x03,0x43,0x68,0xba,0xdc,0x6e,0x22,0x3e,0x16,
  0x6b,0x99,0xb0,0xc6,0xfb,0x44,0xf0,0x45,0xa8,0x09,0xcc,0x17,0x84,0x4a,0xa0,0x9e,
  0x9c,0x89,0xa6,0x00,0xbb,0xac,0xdc,0x12,0xd6,0x06,0xcd,0x10,0xbc,0xb3,0x32,0x83,
  0xe0,0xa0,0xa1,0x8a,0x2d,0x06,0x00,0xe4,0x90,0x11,0x6b,0x06,0x0c,0xb7,0xd7,0xc2,
  0x09,0xc0,0x3d,0x0a,0x80,0xac,0x75,0x72,0xc3,0x78,0x17,0x09,0x00,0x3b
  };

//sid|dat
#define EXT	"*.sid"

// HVSC path to STIL.
#define HVSC_STIL	"/DOCUMENTS/STIL.txt"

// HVSC path to BUGlist.
#define HVSC_BUGLIST	"/DOCUMENTS/BUGlist.txt"

#ifndef ALAN_SIDPLAY
#  define CREDITS	"Sidplay2\nCopyright \302\251 Simon White"
#  define LINK		"http://sourceforge.net/projects/sidplay2/"
#else
#  define CREDITS	"Sidplay2\nCopyright \302\251 Simon White\nModified by Antti Lankila"
#  define LINK		"http://bel.fi/~alankila/c64-sw/"
#endif

const char sid::name[]="Sidplay2";

static const char engines[2][8] = {"reSID", "hardSID"};

/*****************************************************************/

int sid::fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds) {

#ifdef HAVE_SIDUTILS
	if (_length && (seconds>=_length))
		return 0;
#endif

#ifndef ALAN_SIDPLAY
	return _sidplay2->play((void*)buffer, bufferSize);
#else
	const unsigned int size=bufferSize/(_settings.bits>>3);
	float *fBuffer=new float[size];
	FXshort* buf=(FXshort*)buffer;
	const unsigned int n=_sidplay2->play(fBuffer, size);

	for (unsigned int i=0; i<n; i++) {
		for (unsigned int j=0; j<_settings.channels; j++) {
			if (_settings.bits==16)
				buf[i]=_quant->get16((void*)&fBuffer[i], j);
			else
				buffer[i]=_quant->get8((void*)&fBuffer[i], j);
		}
	}
	delete [] fBuffer;
	return n*(_settings.bits>>3);
#endif
}

/*****************************************************************/

bool sid::supports(const char* fileName) {

	return FXPath::match(FXString(EXT), FXPath::name(fileName), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);
}

inline FXString sid::ext() const { return FXString(EXT); }

sid::sid(FXApp* a) :
	inputPlugin(a, name, iconSid),
	_sidplay2(0),
	_stil(0),
#ifdef HAVE_SIDUTILS
	_db(0),
	_length(0),
#endif
	_tune(0)
#ifdef ALAN_SIDPLAY
	,_quant(0)
#endif
 {

	_settings.samplerate=load("Frequency", 44100);
	_settings.channels=load("Channels", 1);
	_settings.bits=load("Bits", 16);
#ifndef ALAN_SIDPLAY
	_settings.optimisation=load("Optimisation", 0);
#endif
	_settings.environment=(sid2_env_t)load("Environment", sid2_envPS);
	_settings.clock=(sid2_clock_t)load("Clock speed", SID2_CLOCK_CORRECT);
	_settings.model=(sid2_model_t)load("SID model", SID2_MODEL_CORRECT);
	_settings.forceClock=load("Force clock", FALSE);
	_settings.engine=load("Engine", engines[0]);
	_settings.filter=load("Filter", TRUE);
	_settings.hvscPath=load("HVSC", FXString::null);
	_settings.psid_filter=load("PSID Filter", TRUE);

	openHvsc(_settings.hvscPath);
}

sid::~sid() {

	close();

	save("Frequency", _settings.samplerate);
	save("Channels", _settings.channels);
	save("Bits", _settings.bits);
#ifndef ALAN_SIDPLAY
	save("Optimisation", _settings.optimisation);
#endif
	save("Environment", _settings.environment);
	save("Clock speed", _settings.clock);
	save("SID model", _settings.model);
	save("Force clock", _settings.forceClock);
	save("Engine", _settings.engine);
	save("Filter", _settings.filter);
	save("HVSC", _settings.hvscPath);
	save("PSID Filter", _settings.psid_filter);
#ifdef HAVE_SIDUTILS
	delete _db;
#endif
	delete _stil;
}

FXint sid::filter(FXString* files, FXint items) {

	if (!_settings.psid_filter)
		return items;

	FXString psids;

	unsigned int j=0;
	for (unsigned int i=0; i<items; i++) {
		if (FXPath::match(ext(), FXPath::name(files[i]), FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD)) {
			files[j++]=files[i];
			int p=files[i].find("_PSID", 5, 0);
			if (p>=0) {
				FXString tmp=files[i];
				psids.append(tmp.erase(p, 5));
			}
		}
	}

	if (!psids.count())
		return j;

	items=0;
	for (unsigned int i=0; i<j; i++) {
		if (psids.find(files[i])<0)
			files[items++]=files[i];
	}

	return items;
}

bool sid::open(const FXString& fileName) {

	close();

	_sidplay2=new sidplay2;
	sidbuilder *emuSid=0;
#ifdef HAVE_SIDPLAY_BUILDERS_RESID_H
	if (!comparecase(_settings.engine, engines[0])) {
		ReSIDBuilder *tmpResid=new ReSIDBuilder("FAP reSID");
		tmpResid->create(_sidplay2->info().maxsids);
#ifndef ALAN_SIDPLAY
		tmpResid->sampling(_settings.samplerate);
#endif
		tmpResid->filter(_settings.filter);
		tmpResid->filter((const sid_filter_t*)0);
		emuSid=(sidbuilder*)tmpResid;
	}
#endif
#ifdef HAVE_SIDPLAY_BUILDERS_HARDSID_H
	if (!comparecase(_settings.engine, engines[1])) {
		HardSIDBuilder *tmpHardsid=new HardSIDBuilder("FAP hardSID");
		tmpHardsid->create(_sidplay2->info().maxsids);
		tmpHardsid->filter(_settings.filter);
		emuSid=(sidbuilder*)tmpHardsid;
	}
#endif
	if (!emuSid) {
		delPtr(_sidplay2);
		return false;
	}

	sid2_config_t cfg;
	cfg=_sidplay2->config();
	cfg.clockForced=_settings.forceClock;
	cfg.clockSpeed=_settings.clock;
	cfg.environment=_settings.environment;
	cfg.frequency=_settings.samplerate;
#ifndef ALAN_SIDPLAY
	cfg.optimisation=_settings.optimisation;
#endif
	cfg.playback=(_settings.channels==2)?sid2_stereo:sid2_mono;
	cfg.sidEmulation=emuSid;
	cfg.sidModel=_settings.model;
	cfg.sidSamples=true;
	_sidplay2->config(cfg);

#ifdef HAVE_SIDUTILS
	_tune=new SidTuneMod(fileName.text());
	_tune->createMD5(_md5);
#else
	_tune=new SidTune(fileName.text());
#endif
	loadTune(0);

	const FX88591Codec codec;
	if (_sidplay2->info().tuneInfo->numberOfInfoStrings==3) {
		addInfo(TITLE, codec.mb2utf(_sidplay2->info().tuneInfo->infoString[0]).text());
		addInfo(ARTIST, codec.mb2utf(_sidplay2->info().tuneInfo->infoString[1]).text());
		addInfo("Released", codec.mb2utf(_sidplay2->info().tuneInfo->infoString[2]).text());
	}

	if (_stil) {
		FXTRACE((10, "Retrieving STIL info\n"));
		FXString comment=FXString(_stil->getAbsGlobalComment(fileName.text()));
		if (!comment.empty())
			comment.append('\n');

		comment.append(FXString(_stil->getAbsEntry(fileName.text())));

		FXString bug=FXString(_stil->getAbsBug(fileName.text()));
		if (!bug.empty()) {
			comment.append('\n');
			comment.append(bug);
		}
		addInfo(COMMENT, codec.mb2utf(comment.text()).text());
	}
#ifdef ALAN_SIDPLAY
	_quant=new requantizeFloat();
#endif
	songLoaded(true);
	return true;
}

void sid::close() {

	delPtr(_tune);
	if (_sidplay2) {
		const sidbuilder *emuSid=_sidplay2->config().sidEmulation;
		delete emuSid;
		delPtr(_sidplay2);
	}
#ifdef ALAN_SIDPLAY
	delPtr(_quant);
#endif
	songLoaded(false);
}

const bool sid::rewind() {

	if (_sidplay2) {
		_sidplay2->stop();
		return true;
	}
	return false;
}

bool sid::subtune(const FXuint i) {

	if (_tune && (i>0) && (i<=_tune->getInfo().songs)) {
		loadTune(i);
		return true;
	}
	return false;
}

const bool sid::config(FXWindow* win) {

	const FXString oldHvscPath=_settings.hvscPath;
	sidConfig cWin(win, _settings);
	if (cWin.execute(PLACEMENT_OWNER)) {
		_settings=cWin.settings();
		if (compare(_settings.hvscPath, oldHvscPath))
			openHvsc(_settings.hvscPath);
		return true;
	}
	return false;
}

void sid::loadTune(const int num) {

	_tune->selectSong(num);
	_sidplay2->load(_tune);
#ifdef HAVE_SIDUTILS
	_length=_db?_db->length(_md5, _tune->getInfo().currentSong):0;
	time(_length);
#endif
}

void sid::openHvsc(const FXString& hvscPath) {

	if (hvscPath.empty())
		return;

	if (!_stil)
		_stil=new STIL(HVSC_STIL, HVSC_BUGLIST);

	if (!_stil->setBaseDir(hvscPath.text())) {
		fxmessage("%s\n", _stil->getErrorStr());
		delPtr(_stil);
	}

#ifdef HAVE_SIDUTILS
	if (!_db)
		_db=new SidDatabase();

	if (_db->open(FXStringFormat("%s%cDOCUMENTS%cSonglengths.txt", hvscPath.text(), PATHSEP, PATHSEP).text())) {
		fxmessage("%s\n", _db->error());
		delPtr(_db);
	}
#endif
}

/*****************************************************************/

FXDEFMAP(sidConfig) sidConfigMap[] = {
	// Message_Type		ID				Message_Handler
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_FREQUENCY,	sidConfig::onCmdFrequency),
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_CHANNELS,		sidConfig::onCmdChannels),
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_BITS,		sidConfig::onCmdBits),
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_ENGINE,		sidConfig::onCmdEngine),
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_ENVIRONMENT,	sidConfig::onCmdEnvironment),
	FXMAPFUNC(SEL_COMMAND,	sidConfig::ID_HVSC,		sidConfig::onCmdHvsc),
};

FXIMPLEMENT(sidConfig, configWindow, sidConfigMap, ARRAYNUMBER(sidConfigMap))

sidConfig::sidConfig(FXWindow* win, sidConfig_t settings) :
	configWindow(win, sid::name, CREDITS, LINK),
	_settings(settings) {

	FXint val;

#ifndef ALAN_SIDPLAY
	_optimisation.connect(_settings.optimisation);
#endif
	_filter.connect(_settings.filter);
	_hvscDir.connect(_settings.hvscPath);
	_psid_filter.connect(_settings.psid_filter);

	new FXLabel(matrix(), tr("Frequency"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *freqBox=new FXListBox(matrix(), this, sidConfig::ID_FREQUENCY, LAYOUT_FILL|FRAME_NORMAL);
	freqBox->fillItems("11025\n22050\n44100\n48000");
	freqBox->setNumVisible(4);
	switch (_settings.samplerate) {
	case 11025:
		val=0;
		break;
	case 22050:
		val=1;
		break;
	default:
	case 44100:
		val=2;
		break;
	case 48000:
		val=3;
		break;
	}
	freqBox->setCurrentItem(val);

	new FXLabel(matrix(), tr("Channels"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *chanBox=new FXListBox(matrix(), this, sidConfig::ID_CHANNELS, LAYOUT_FILL|FRAME_NORMAL);
	chanBox->fillItems("Mono\nStereo");
	chanBox->setNumVisible(2);
	chanBox->setCurrentItem(_settings.channels-1);

	new FXLabel(matrix(), tr("Bits"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *_bitBox=new FXListBox(matrix(), this, sidConfig::ID_BITS, LAYOUT_FILL|FRAME_NORMAL);
	_bitBox->fillItems("8\n16");
	_bitBox->setNumVisible(2);
	switch (_settings.bits) {
	case 8:
		val=0;
		break;
	default:
	case 16:
		val=1;
		break;
	}
	_bitBox->setCurrentItem(val);

	new FXLabel(matrix(), tr("Environment"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *resBox=new FXListBox(matrix(), this, sidConfig::ID_ENVIRONMENT, LAYOUT_FILL|FRAME_NORMAL);
	resBox->fillItems("Playsid\nTransparent Rom\nBankswitching\nReal C64 Environment");
	resBox->setNumVisible(4);
	switch (_settings.environment) {
	default:
	case sid2_envPS:
		val=0;
		break;
	case sid2_envTP:
		val=1;
		break;
	case sid2_envBS:
		val=2;
		break;
	case sid2_envR:
		val=3;
		break;
	case sid2_envTR:
		val=4;
		break;
	}
	resBox->setCurrentItem(val);

	new FXLabel(matrix(), tr("Engine"), 0, LAYOUT_FILL|JUSTIFY_RIGHT);
	FXListBox *engBox=new FXListBox(matrix(), this, sidConfig::ID_ENGINE, LAYOUT_FILL|FRAME_NORMAL);
#ifdef HAVE_SIDPLAY_BUILDERS_RESID_H
	engBox->appendItem(engines[0]);
#endif

#ifdef HAVE_SIDPLAY_BUILDERS_HARDSID_H
	engBox->appendItem(engines[1]);
#endif
	const FXint numItems=engBox->getNumItems();
	engBox->setNumVisible(numItems);
	const FXint curItem=engBox->findItem(_settings.engine);
	if (curItem>=0)
		engBox->setCurrentItem(curItem);

	new FXVerticalSeparator(extraLeft());

	FXVerticalFrame *vert=new FXVerticalFrame(extraLeft());
#ifndef ALAN_SIDPLAY
	new FXCheckButton(vert, tr("Optimisation\tFaster but inaccurate emulation"), &_optimisation, FXDataTarget::ID_VALUE);
#endif
	new FXCheckButton(vert, tr("Filter\tEmulate SID filter"), &_filter, FXDataTarget::ID_VALUE);
	new FXCheckButton(vert, tr("Playlist filter\tDon't show SIDs for which an PSID version exists"),
		&_psid_filter, FXDataTarget::ID_VALUE);

	FXHorizontalFrame *hf=new FXHorizontalFrame(extraBottom());
	new FXLabel(hf, tr("HVSC path:"));
	new FXTextField(hf, 40, &_hvscDir, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|TEXTFIELD_NORMAL);
	new FXButton(hf, tr("&Browse\tSelect HVSC directory"), 0, this, sidConfig::ID_HVSC, FRAME_RAISED|FRAME_THICK|JUSTIFY_LEFT|ICON_BEFORE_TEXT);
}

long sidConfig::onCmdFrequency(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		_settings.samplerate=11025;
		break;
	case 1:
		_settings.samplerate=22050;
		break;
	case 2:
		_settings.samplerate=44100;
		break;
	case 3:
		_settings.samplerate=48000;
		break;
	}
	return 1;
}

long sidConfig::onCmdChannels(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	_settings.channels=val+1;
	return 1;
}

long sidConfig::onCmdBits(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		_settings.bits=8;
		break;
	case 1:
		_settings.bits=16;
		break;
	}
	return 1;
}

long sidConfig::onCmdEngine(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	_settings.engine=engines[val];
	return 1;
}

long sidConfig::onCmdEnvironment(FXObject*, FXSelector, void* ptr) {

	const FXint val=(FXint)(FXival)ptr;
	switch (val) {
	case 0:
		_settings.environment=sid2_envPS;
		break;
	case 1:
		_settings.environment=sid2_envTP;
		break;
	case 2:
		_settings.environment=sid2_envBS;
		break;
	case 3:
		_settings.environment=sid2_envR;
		break;
	}
	return 1;
}

long sidConfig::onCmdHvsc(FXObject*, FXSelector, void*) {

	FXDirDialog dirDialog(this, tr("Select HVSC directory"));
	if (dirDialog.execute())
		_settings.hvscPath=dirDialog.getDirectory();

	return 1;
}

#endif
