/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SID_H
#define SID_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#if defined (HAVE_SIDPLAY_BUILDERS_RESID_H) || defined (HAVE_SIDPLAY_BUILDERS_HARDSID_H)

#include <sidplay/sidplay2.h>

#define HAVE_SIDPLAY2

#ifdef HAVE_SIDUTILS
# include <sidplay/utils/SidDatabase.h>
#endif

#include <fx.h>
#include "inputPlugin.h"
#include "stil.h"

#ifdef ALAN_SIDPLAY
#  include "requantize.h"
#endif

/*****************************************************************/

typedef struct {
	FXString	engine;
	FXint		samplerate;
	FXint		channels;
	FXint		bits;
#ifndef ALAN_SIDPLAY
	FXint		optimisation;
#endif
	sid2_env_t	environment;
	sid2_clock_t	clock;
	FXbool		forceClock;
	sid2_model_t	model;
	FXbool		filter;
	FXString	hvscPath;
	FXbool		psid_filter;
} sidConfig_t;

/*****************************************************************/

class sid : public inputPlugin {

private:
	sidplay2	*_sidplay2;

	sidConfig_t	_settings;

	STIL		*_stil;

#ifdef HAVE_SIDUTILS
	SidDatabase	*_db;
	FXuint		_length;
	char		_md5[SIDTUNE_MD5_LENGTH+1];

	SidTuneMod	*_tune;
#else
	SidTune		*_tune;
#endif

#ifdef ALAN_SIDPLAY
	requantize	*_quant;
#endif

private:
	void loadTune(const int num);

	void openHvsc(const FXString& hvscPath);

public:
	sid(FXApp* a);
	~sid();

	static const char name[];

	/// Factory method
	static input* factory(FXApp *a) { return new sid(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Filter dir content
	FXint filter(FXString* files, FXint items);

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Number of subtunes
	FXuint subtunes() const { return _tune?_tune->getInfo().songs:0; }

	/// Current subtune
	FXuint subtune() const { return _tune?_tune->getInfo().currentSong:0; }

	/// Change subtune
	bool subtune(const FXuint i);

	/// Get samplerate
	const FXint samplerate() const { return _tune?_settings.samplerate:0; }

	/// Get channels
	const FXint channels() const { return _tune?_settings.channels:0; }

	/// Get precision
	const FXint precision() const { return _tune?(_settings.bits>>3):0; }

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);
};

/*****************************************************************/

#include "configWindow.h"

class sidConfig : public configWindow {
	FXDECLARE(sidConfig)

private:
	sidConfig_t		_settings;
#ifndef ALAN_SIDPLAY
	FXDataTarget		_optimisation;
#endif
	FXDataTarget		_filter;
	FXDataTarget		_hvscDir;
	FXDataTarget		_psid_filter;

private:
	sidConfig() {};
	sidConfig(const sidConfig&);
	sidConfig& operator=(const sidConfig&);

public:
	sidConfig(FXWindow* win, sidConfig_t settings);
	virtual ~sidConfig() {};

	/// Get settings
	sidConfig_t settings() const { return _settings; }

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
		ID_BITS,
		ID_ENGINE,
		ID_ENVIRONMENT,
		ID_HVSC,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
	long onCmdBits(FXObject*, FXSelector, void*);
	long onCmdEngine(FXObject*, FXSelector, void*);
	long onCmdEnvironment(FXObject*, FXSelector, void*);
	long onCmdHvsc(FXObject*, FXSelector, void*);
};
#endif

#endif
