/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SND_H
#define SND_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_SNDFILE
#include <sndfile.h>

#include "inputPlugin.h"
#include <fx.h>

/*****************************************************************/

class snd : public inputPlugin {

private:
	SNDFILE		*_sf;
	SF_INFO		_si;

	static FXString		_ext;

public:
	snd(FXApp *a) :
		inputPlugin(a, name),
		_sf(0) {};
	~snd();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory(FXApp *a) { return new snd(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const { return _ext; }

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const { return _sf?_si.samplerate:0; }

	/// Get channels
	const FXint channels() const { return _sf?_si.channels:0; }

	/// Get precision
	const FXint precision() const { return 2; }

	/// Callback function
	int fillBuffer(char *buffer, const unsigned int bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; };
};

/*****************************************************************/

#include "configWindow.h"

class sndConfig : public configWindow {
	FXDECLARE(sndConfig)

private:
	sndConfig() {};
	sndConfig(const sndConfig&);
	sndConfig& operator=(const sndConfig&);

public:
	sndConfig(FXWindow* win);
	virtual ~sndConfig() {};
};
#endif

#endif
