/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TTA_H
#define TTA_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_TTA
#include <stdio.h>
extern "C" {
#include "ttalib-hwplayer-1.2/ttalib.h"
}

#include "inputPlugin.h"
#include <fx.h>

/*****************************************************************/

class tta : public inputPlugin {

private:
	tta_info	_ttaInfo;
	FXuint		_sampleSize;
	FXuchar		*_decodeBuf;
	FXint		_bufOffset;
	FXint		_bufSize;

public:
	tta(FXApp* a);
	~tta();

	static const char name[];

	/// Factory method
	static input* factory(FXApp *a) { return new tta(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const {
		return songLoaded()?
			_ttaInfo.SAMPLERATE:
			0;
	}

	/// Get channels
	const FXint channels() const {
		return songLoaded()?
			_ttaInfo.NCH:
			0;
	}

	/// Get precision
	const FXint precision() const {
		return songLoaded()?
			_ttaInfo.BPS>>3:
			0;
	}

	/// Callback function
	int fillBuffer(char* buffer, const unsigned int bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; };
};

/*****************************************************************/

#include "configWindow.h"

class ttaConfig : public configWindow {
	FXDECLARE(ttaConfig)

private:
	ttaConfig() {};
	ttaConfig(const ttaConfig&);
	ttaConfig& operator=(const ttaConfig&);

public:
	ttaConfig(FXWindow* win);
	virtual ~ttaConfig() {};
};

#endif

#endif
