/*
 *  Copyright (C) 2007-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef WV_H
#define WV_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_WAVPACK
extern "C" {
#include <wavpack/wavpack.h>
}

#include "inputPlugin.h"
#include "requantize.h"
#include <fx.h>

/*****************************************************************/

class wv : public inputPlugin {

private:
	WavpackContext	*_wvContext;
	FXint		*_decodeBuf;
	FXint		_bufOffset;
	FXint		_bufSize;
	FXint		_bps;
	FXint		_channels;
	FXint		_precision;

	requantize	*_quant;

private:
	void copyBuffer(FXchar* dest, const FXint* src, unsigned int length);

public:
	wv(FXApp* a);
	~wv();

	static const char name[];

	/// Factory method
	static input* factory(FXApp *a) { return new wv(a); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Open config dialog
	const bool config(FXWindow* win);

	/// Get samplerate
	const FXint samplerate() const {
		return _wvContext?
			WavpackGetSampleRate(_wvContext):
			0;
	}

	/// Get channels
	const FXint channels() const {
		return _wvContext?
			WavpackGetReducedChannels(_wvContext):
			0;
	}

	/// Get precision
	const FXint precision() const {
		return _wvContext?
			_precision:
			0;
	}

	/// Callback function
	int fillBuffer(char* buffer, unsigned int bufferSize, unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; };
};

/*****************************************************************/

#include "configWindow.h"

class wvConfig : public configWindow {
	FXDECLARE(wvConfig)

private:
	wvConfig() {};
	wvConfig(const wvConfig&);
	wvConfig& operator=(const wvConfig&);

public:
	wvConfig(FXWindow* win);
	virtual ~wvConfig() {};
};

#endif

#endif
