/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ALSA_H
#define ALSA_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_ALSA

#include <fx.h>
#include <alsa/asoundlib.h>

#include "outputPlugin.h"

/*****************************************************************/

/**
 * ALSA output backend
*/
class alsa : public outputPlugin {

private:
	snd_pcm_t		*_pcmHandle;
	snd_mixer_t		*_mixer;
	snd_mixer_elem_t	*_mixerHandle;

	char			*_buffer;

private:
	const int setParams(int &sampleRate, const int channels, const int prec);

	bool initMixer(const char* dev);

public:
	alsa(FXApp *app);
	virtual ~alsa() {};

	static const char name[];

	/// Factory method
	static output* factory(FXApp* a) { return new alsa(a); }

	/// Open
	int open(const int card, int &sampleRate, const int channels, const int prec);

	/// Close
	void close();

	/// Get buffer
	char *buffer() { return _buffer; }

	/// Write data to output
	bool write(char* buffer, int bufferSize);

	/// Set volume
	void volume(int vol);
};

#endif

#endif
