/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef OUTPUT_PLUGIN_H
#define OUTPUT_PLUGIN_H

#include "output.h"

#include <new>
#include <FXArray.h>

/*****************************************************************/

class outputPlugin : public output {

private:
	FXApp		*_app;
	const char	*_name;

	FXArray<FXString>	_devices;

private:
	outputPlugin();
	outputPlugin(const outputPlugin&);
	outputPlugin& operator=(const outputPlugin&);

protected:
	outputPlugin(FXApp* app, const char name[]) : _app(app), _name(name) {};

	/// Add a device to the list
	void addDevice(const FXchar* dev) { _devices.append(dev); }

	/// Load int setting
	FXint load(const FXchar* key, FXint defVal) {
		return _app->reg().readIntEntry("Audio Settings", key, defVal);
	}

	/// Load string setting
	FXString load(const FXchar* key, FXString defVal) {
		return FXString(_app->reg().readStringEntry("Audio Settings", key, defVal.text()));
	}

	/// Save int setting
	void save(const FXchar* key, FXint value) {
		_app->reg().writeIntEntry("Audio Settings", key, value);
	}

	/// Save string setting
	void save(const FXchar* key, FXString value) {
		_app->reg().writeStringEntry("Audio Settings", key, value.text());
	}

public:
	virtual ~outputPlugin() {};

	/// Get backend name
	const char* name() const { return _name; }

	/// Get number of available devices
	const int devices() const { return _devices.no(); }

	/// Get device name
	const FXString device(const int i) const { return _devices[i]; }

	/// Pause
	virtual void pause() {};

	/// Set volume
	virtual void volume(int vol) {};
};

#endif
