/*
 *  Copyright (C) 2006-2007 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef REQUANTIZE_H
#define REQUANTIZE_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <fx.h>

class requantize {

protected:
	FXuint	_random[2][2];

public:
	virtual ~requantize() {};

	/// Get 8 bit data
	virtual FXchar get8(void* sample, const unsigned int channel)=0;

	/// Get 16 bit data
	virtual FXshort get16(void* sample, const unsigned int channel)=0;

	/// Get 24 bit data
	//virtual FXint get24(void* sample)=0;
};

/*************************************************************************/

class requantizeInt : public requantize {

private:
	FXint	_scalebits;
	FXint	_clip;
	FXint	_mask;

private:
	requantizeInt();
	requantizeInt(const requantizeInt&);
	requantizeInt& operator=(const requantizeInt&);

	FXint get32(FXint sample, const unsigned int channel);

public:
	requantizeInt(FXuint precision, FXuint fract);
	~requantizeInt() {};

	/// Get 8 bit data
	FXchar get8(void* sample, const unsigned int channel);

	/// Get 16 bit data
	FXshort get16(void* sample, const unsigned int channel);
};

/*************************************************************************/

class requantizeFloat : public requantize {

private:
	requantizeFloat(const requantizeFloat&);
	requantizeFloat& operator=(const requantizeFloat&);

	FXint get32(FXfloat sample, const unsigned int channel, const FXint max);

public:
	requantizeFloat();
	~requantizeFloat() {};

	/// Get 8 bit data
	FXchar get8(void* sample, const unsigned int channel);

	/// Get 16 bit data
	FXshort get16(void* sample, const unsigned int channel);
};

#endif
