/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

// Based on code posted by Rafael de Pelegrini Soares on the Fox-toolkit mailing list

#include "LinkLabel.h"

#ifdef _WIN32
#  include <windows.h>
#endif

FXDEFMAP(FXLinkLabel) FXLinkLabelMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXLinkLabel::onLeftBtnPress),
  FXMAPFUNC(SEL_ENTER,0,FXLinkLabel::onEnter),
  FXMAPFUNC(SEL_LEAVE,0,FXLinkLabel::onLeave),
  FXMAPFUNC(SEL_TIMEOUT,FXLinkLabel::ID_TIMER,FXLinkLabel::onTimer),
};

FXIMPLEMENT(FXLinkLabel,FXLabel,FXLinkLabelMap,ARRAYNUMBER(FXLinkLabelMap))

bool FXLinkLabel::fxexecute(const FXString& link){
#ifdef _WIN32
#ifdef UNICODE
  FXint n=ncslen(link.text(),link.length());
  FXnchar *buffer=new FXnchar[n+1];
  utf2ncs(buffer,link.text(),link.length()+1);
  n=((int)ShellExecute(NULL,L"open",buffer,NULL,NULL,SW_SHOW)>32);
  delete [] buffer;
  return n;
#else
  return ((int)ShellExecute(NULL,"open",link.text(),NULL,NULL,SW_SHOW)>32);
#endif
#else
  const FXString ext=FXPath::extension(link);
  FXString list=FXString::null;
  if (!comparecase(link.section(':',0),"http") ||
    !comparecase(link.section(':',0),"ftp") ||
    !comparecase(ext,"htm") || !comparecase(ext,"html") ||
    !comparecase(ext,"php") || !comparecase(ext,"asp"))
    list="opera\tkonqueror\tmozilla-firefox\tmozilla\tnetscape\tdillo\tlynx";
  else if (!comparecase(ext,"pdf"))
    list="kpdf\tacroread\tkghostview\tgpdf\txpdf";

 if (!list.empty()){
    FXint index=0;
    const FXString path=FXSystem::getExecPath();

    FXString software=list.section("\t",index);
    while (!software.empty()){
      software=FXPath::search(path,software);
      if (!software.empty())
        return (system(FXStringFormat("%s \"%s\" >/dev/null 2>&1 & ",software.text(),link.text()).text())>0);
      software=list.section("\t",++index);
      }
    }
  else if (FXStat::isExecutable(link))
    return (system((link+" >/dev/null 2>&1 & ").text())>0);
  return false;
#endif
}

FXLinkLabel::FXLinkLabel(FXComposite* p,const FXString& text,FXIcon* ic,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXLabel(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){
  setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
  setTextColor(FXRGB(0,0,255));
}

FXLinkLabel::~FXLinkLabel(){
  getApp()->removeTimeout(this,ID_TIMER);
}

long FXLinkLabel::onLeftBtnPress(FXObject*,FXSelector,void*){
  const FXString link=getText();
  if (!link.empty()) {
    getApp()->beginWaitCursor();
    if (fxexecute(link))
      getApp()->addTimeout(this,ID_TIMER,2000); // 2 seconds of wait cursor
    else {
      getApp()->endWaitCursor();
      getApp()->beep();
      }
    }
  return 1;
}

long FXLinkLabel::onEnter(FXObject*,FXSelector,void*){
  setTextColor(FXRGB(255,0,0));
  return 1;
}

long FXLinkLabel::onLeave(FXObject*,FXSelector,void*){
  setTextColor(FXRGB(0,0,255));
  return 1;
}

long FXLinkLabel::onTimer(FXObject*,FXSelector,void*){
  getApp()->endWaitCursor();
  return 1;
}
