/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "infoDialog.h"
#include "LinkLabel.h"
#include "settings.h"
#include "iconFactory.h"

#if FOX_LEVEL<32
#  include <FXJPGImage.h>
#endif

FXIMPLEMENT(textLabel, FXLabel, 0, 0)

/*****************************************************************/

FXIMPLEMENT(infoDialog, FXDialogBox, 0, 0)

infoDialog::infoDialog(FXWindow* win, input* i, const char* folder) :
	FXDialogBox(win, FXString::null, DECOR_TITLE|DECOR_BORDER|DECOR_STRETCHABLE),
	_font(0),
	_img(0) {

	setTitle(tr("Song Info"));

	new FXButton(this, tr("&Close"), GET_ICON(getApp(), icon_dialogcancel), this, FXDialogBox::ID_CANCEL,
		BUTTON_INITIAL|BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_SIDE_BOTTOM);
	new FXSeparator(this, SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

	if (!i->songLoaded()) {
		new FXLabel(this, tr("No song loaded"));
		return;
	}

	FXMatrix* matrix=new FXMatrix(this, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X);

	int j=-1;
	while ((j=i->moreInfo(j))>=0) {
		FXString temp=i->getInfo(j);
		if (temp.empty())
			continue;

		FXint rows=1;
		FXint cols=0;
		FXint cnt=0;
		FXint pos=0;
		const FXint len=temp.length();
		const FXchar* s=temp.text();
		while (pos<len) {
			// Convert Mac and Windows EOLs to Unix
			if (s[pos]=='\r')
				temp.replace(pos, (s[pos+1]=='\n')?'\0':'\n');
			//Count rows and columns
			if (s[pos]=='\n') {
				rows++;
				if (cnt>cols)
					cols=cnt;
				cnt=0;
			} else
				cnt++;
			pos+=FXString::utfBytes[(FXuchar)s[pos]];
		}

		FXTRACE((10, "Rows: %d\n", rows));
		FXTRACE((10, "Cols: %d\n", cols));

		if (rows>1) {
			_font=new FXFont(getApp(), "courier", 10,
				FXFont::Normal, FXFont::Straight,
				FONTENCODING_DEFAULT, FXFont::NonExpanded, FXFont::Fixed);
			FXHorizontalFrame *frame=new FXHorizontalFrame(this, LAYOUT_FILL|FRAME_NORMAL, 0,0,0,0, 0,0,0,0, 0,0);
			FXText *text=new FXText(frame, 0, 0, TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL);
			text->setFont(_font);
			text->setText(temp);
			text->setVisibleRows((rows<20)?rows:20);
			text->setVisibleColumns((cols<80)?cols+2:80);
		} else {
			new FXLabel(matrix, FXStringFormat("%s:", i->getKey(j)), 0, LAYOUT_FILL_X|JUSTIFY_RIGHT);
			if (FXRex("\\A(?ihttp://)?(?iwww\\.)?[-a-zA-Z_0-9/~\\.]+\\.\\l{2,4}/?\\Z").match(temp))
				new FXLinkLabel(matrix, temp);
			else
				new textLabel(matrix, temp);
		}
	}

	if (SETTINGS->albumArt()) {
		// Display album art
		FXIconSource iconsource(getApp());
		FXString* files=0;
		FXint n=FXDir::listFiles(files, folder, "(cover|front|folder).*", FXDir::CaseFold);
		while (n && !_img) {
			FXString file=FXStringFormat("%s%c%s", folder, PATHSEP, files[--n].text());
			FXTRACE((10, "Trying to load album art: %s\n", file.text()));
#if FOX_LEVEL<32
			// Hack to overcome a FOX bug
			if (FXJPGImage::supported && !comparecase(FXPath::extension(file), "jpeg"))
				_img=iconsource.loadScaledImageFile(file, SETTINGS->aaSize(), 0, FXJPGImage::fileExt);
			else
#endif
				_img=iconsource.loadScaledImageFile(file, SETTINGS->aaSize());
		}

		delete [] files;

		if (_img) {
			_img->create();
			new FXImageFrame(this, _img, LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK);
		}
	}

	if (!matrix->getFirst())
		new FXLabel(this, tr("No info"), 0, LAYOUT_FILL_X);
}

infoDialog::~infoDialog() {

	delete _img;
	delete _font;
}
