/*
 *  Copyright (C) 2006-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "mainWindow.h"
#include "AboutDialog.h"
#include "infoDialog.h"
#include "utils.h"
#include "settings.h"
#include "iconFactory.h"
#include "factory.h"

#include "ledRed.xpm"
#include "ledYellow.xpm"
#include "ledGreen.xpm"

/*****************************************************************/

FXint loadThread::run() {

	FXint res=_i->open(_name);
	_loaded->setData((void*)(FXival)res);
	_loaded->signal();
	return 0;
}

/*****************************************************************/

FXDEFMAP(mainWindow) mainWindowMap[] = {
	// Message_Type			ID				Message_Handler
	FXMAPFUNC(SEL_IO_READ,		mainWindow::ID_AUDIO_MSG,	mainWindow::onAudioMsg),
	FXMAPFUNC(SEL_UPDATE,		mainWindow::ID_FILETYPE,	mainWindow::onUpdFiletype),
	FXMAPFUNC(SEL_UPDATE,		mainWindow::ID_LED,		mainWindow::onUpdLed),
	FXMAPFUNC(SEL_UPDATE,		mainWindow::ID_PLSONG,		mainWindow::onUpdSubtune),
	FXMAPFUNC(SEL_UPDATE,		mainWindow::ID_SUBTUNE,		mainWindow::onUpdSubtune),
	FXMAPFUNC(SEL_CHANGED,		mainWindow::ID_DIRLIST,		mainWindow::onCmdDirList),
	FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,mainWindow::ID_DIRLIST,	mainWindow::onRgtClkDirList),
	FXMAPFUNC(SEL_DOUBLECLICKED,	mainWindow::ID_PLAYLIST,	mainWindow::onCmdPlaySong),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_PLAY,		mainWindow::onCmdPlaySong),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_STOP,		mainWindow::onCmdStopSong),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_PAUSE,		mainWindow::onCmdPauseSong),
	FXMAPFUNCS(SEL_COMMAND,		mainWindow::ID_PREV,mainWindow::ID_NEXT,	mainWindow::onCmdChangeSong),
	FXMAPFUNC(SEL_SELECTED,		mainWindow::ID_PLAYLIST,	mainWindow::onCmdSongSelected),
	FXMAPFUNC(SEL_IO_READ,		mainWindow::ID_SONG_LOADED,	mainWindow::onCmdSongLoaded),
	FXMAPFUNCS(SEL_COMMAND,	mainWindow::ID_PREV_SUBTUNE,mainWindow::ID_NEXT_SUBTUNE,	mainWindow::onCmdSubtune),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_BOOKMARKS,	mainWindow::onCmdBmGet),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_AUDIO_CONFIG,	mainWindow::onCmdConfig),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_OPT_CONFIG,	mainWindow::onCmdConfig),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_INPUT_CONFIG,	mainWindow::onCmdInputConfig),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_FILETYPE,	mainWindow::onCmdFiletype),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_VOL,		mainWindow::onCmdVol),
	FXMAPFUNC(SEL_CHANGED,		mainWindow::ID_VOL,		mainWindow::onCmdVol),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_UI_MODE,		mainWindow::onCmdUIMode),
	FXMAPFUNC(SEL_UPDATE,		mainWindow::ID_INFO_DISPLAY,	mainWindow::onUpdInfoDisplay),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_PL_EDIT,		mainWindow::onCmdPlEdit),
	FXMAPFUNC(SEL_COMMAND,		mainWindow::ID_PL_SAVE,		mainWindow::onCmdPlSave),
	FXMAPFUNCS(SEL_COMMAND,		mainWindow::ID_BM_ADD,mainWindow::ID_PL_ADD,	mainWindow::onCmdAddItem),
	FXMAPFUNCS(SEL_COMMAND,		mainWindow::ID_INFO,mainWindow::ID_ABOUT,	mainWindow::onCmdInfo),
	FXMAPFUNC(SEL_IO_READ,		mainWindow::ID_MESSAGE,		mainWindow::onMessage),
};

FXIMPLEMENT(mainWindow, FXMainWindow, mainWindowMap, ARRAYNUMBER(mainWindowMap))

mainWindow::mainWindow(FXApp* a, const char* lockFileName) :
	FXMainWindow(a, PACKAGE_NAME),
	_input(0),
	_preload(0),
	_loadThread(0),
	_playing(false),
	_preloaded(FXString::null),
	_curSong(FXString::null),
	_playDir(FXString::null),
	_lockFile(0),
	_lockFileName(lockFileName) {

	ledRedIcon=new FXXPMIcon(a, ledRed);
	ledYellowIcon=new FXXPMIcon(a, ledYellow);
	ledGreenIcon=new FXXPMIcon(a, ledGreen);

	setIcon(GET_ICON(a, icon_fap32));
	setMiniIcon(GET_ICON(a, icon_fap16));

	_audio=new audio(a, this, mainWindow::ID_AUDIO_MSG);

	_toolbar=new FXToolBar(this, LAYOUT_FILL_X);
	FXToolBar* toolbar2=new FXToolBar(this, LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_X);
	_statusbar=new FXStatusBar(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
	_statusbar->getStatusLine()->setNormalText(FXString(PACKAGE_STRING));

	_cfgMenu=new FXMenuPane(this, POPUP_SHRINKWRAP);
	new FXMenuCommand(_cfgMenu, tr("Audio\t\tAudio output config"), GET_ICON(a, icon_card),
		this, mainWindow::ID_AUDIO_CONFIG);
	new FXMenuCommand(_cfgMenu, tr("Options\t\tProgram options"), GET_ICON(a, icon_preferencesdesktop),
		this, mainWindow::ID_OPT_CONFIG);

	// toolbar buttons
	new FXButton(_toolbar, tr("\tPrevious\tGo to previous song"), GET_ICON(a, icon_mediaskipbackward),
		this, mainWindow::ID_PREV, BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(_toolbar, tr("\tStop\tStop playing"), GET_ICON(a, icon_mediaplaybackstop),
		this, mainWindow::ID_STOP, BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(_toolbar, tr("\tPlay\tPlay selected song"), GET_ICON(a, icon_mediaplaybackstart),
		this, mainWindow::ID_PLAY, BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(_toolbar, tr("\tPause\tPause"), GET_ICON(a, icon_mediaplaybackpause),
		this, mainWindow::ID_PAUSE, BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(_toolbar, tr("\tNext\tGo to next song"), GET_ICON(a, icon_mediaskipforward),
		this, mainWindow::ID_NEXT, BUTTON_TOOLBAR|FRAME_RAISED);
	_timeDisplay=new FX7Segment(_toolbar, "--:--", SEVENSEGMENT_SHADOW|FRAME_NORMAL);
	_timeDisplay->setTarget(this);
	_timeDisplay->setSelector(ID_TIME);
	new FXToggleButton(_toolbar, tr("\tCompact mode\tSwitch to compact UI mode"),
		tr("\tFull mode\tSwitch to full UI mode"), GET_ICON(a, icon_viewcompact), GET_ICON(a, icon_viewfull),
		this, mainWindow::ID_UI_MODE, TOGGLEBUTTON_NORMAL|TOGGLEBUTTON_TOOLBAR);
	new FXMenuButton(_toolbar, tr("\tConfig\tConfigure"), GET_ICON(a, icon_preferencesdesktop),
		_cfgMenu, MENUBUTTON_TOOLBAR|FRAME_RAISED);

	// toolbar
	new FXButton(toolbar2, tr("\tPrevious subtune\tGo to previous subtune"), GET_ICON(a, icon_goprevious),
		this, mainWindow::ID_PREV_SUBTUNE, BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_CENTER_Y);
	FXLabel* tmpLabel=new FXLabel(toolbar2, "--/--\t\tSubtune", 0, FRAME_SUNKEN|LAYOUT_CENTER_Y);
	tmpLabel->setTarget(this);
	tmpLabel->setSelector(ID_SUBTUNE);
	new FXButton(toolbar2, tr("\tNext subtune\tGo to next subtune"),  GET_ICON(a, icon_gonext),
		this, mainWindow::ID_NEXT_SUBTUNE, BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_CENTER_Y);

	new FXVerticalSeparator(toolbar2, SEPARATOR_NONE);

	tmpLabel=new FXLabel(toolbar2, FXString::null, ledRedIcon, LABEL_NORMAL|LAYOUT_CENTER_Y);
	tmpLabel->setTarget(this);
	tmpLabel->setSelector(ID_LED);
	tmpLabel->setHelpText(tr("Stop"));

	tmpLabel=new FXLabel(toolbar2, "--/--\t\tSong", 0, FRAME_SUNKEN|LAYOUT_CENTER_Y);
	tmpLabel->setTarget(this);
	tmpLabel->setSelector(ID_PLSONG);

	_playMode=new FXToggleButton(toolbar2, tr("\tPlaylist mode\tSwitch to playlist mode"),
		tr("\tSong mode\tSwitch to song mode"), GET_ICON(a, icon_song), GET_ICON(a, icon_playlist),
		0, 0, TOGGLEBUTTON_NORMAL|TOGGLEBUTTON_TOOLBAR|LAYOUT_CENTER_Y);
	FXKnob* volume=new FXKnob(toolbar2, this, mainWindow::ID_VOL, KNOB_NORMAL|LAYOUT_RIGHT);
	volume->setHelpText(tr("Volume"));
	volume->setValue(_audio->volume());

	// statusbar buttons
	new FXButton(_statusbar, tr("\tAbout\tAbout"), GET_ICON(a, icon_fap16),
		this, mainWindow::ID_ABOUT, BUTTON_TOOLBAR|FRAME_RAISED);
	new FXButton(_statusbar, tr("\tExit\tExit"), GET_ICON(a, icon_applicationexit),
		this, ID_CLOSE, BUTTON_TOOLBAR|FRAME_RAISED);

	//
	FXHorizontalFrame* hframe=new FXHorizontalFrame(this, LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
	new FXButton(hframe, tr("\tInfo\tSong info"), GET_ICON(a, icon_helpabout),
		this, mainWindow::ID_INFO, BUTTON_TOOLBAR|FRAME_RAISED);
	_songInfo=new FXTextField(hframe, 30, this, ID_INFO_DISPLAY, TEXTFIELD_READONLY|LAYOUT_FILL|FRAME_SUNKEN);
	_songTime=new FXLabel(hframe, "00:00", 0, FRAME_SUNKEN);

	_mainView=new FXHorizontalFrame(this, LAYOUT_FILL|PACK_UNIFORM_WIDTH, 0,0,0,0, 0,0,0,0, 0,0);

	// Left view - file list/bookmarks
	FXVerticalFrame* frame=new FXVerticalFrame(_mainView, FRAME_NORMAL|LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
	FXHorizontalFrame* buttons=new FXHorizontalFrame(frame, LAYOUT_FILL_X|PACK_UNIFORM_WIDTH, 0,0,0,0, 0,0,0,0, 0,0);
	FXSwitcher* switcher=new FXSwitcher(frame, LAYOUT_FILL|FRAME_NONE, 0,0,0,0, 0,0,0,0);
	hframe=new FXHorizontalFrame(frame, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0, 0,0);
	_fileTypes=new FXListBox(hframe, this, mainWindow::ID_FILETYPE, LAYOUT_FILL_X|FRAME_NORMAL);
	_fileTypes->setHelpText(tr("Select backend"));
	_editMode=new FXToggleButton(hframe, tr("\t\tEdit playlist"), "", GET_ICON(a, icon_editlist),
		GET_ICON(a, icon_editlist), this, mainWindow::ID_PL_EDIT, TOGGLEBUTTON_KEEPSTATE|TOGGLEBUTTON_NORMAL);
	new FXButton(hframe, tr("\t\tSave playlist"), GET_ICON(a, icon_documentsave),
		this, mainWindow::ID_PL_SAVE, BUTTON_NORMAL);

	new FXButton(buttons, tr("Playlist"), GET_ICON(a, icon_playlist), switcher,
		FXSwitcher::ID_OPEN_FIRST, BUTTON_NORMAL|LAYOUT_FILL_X);
	new FXButton(buttons, tr("&Bookmarks"), GET_ICON(a, icon_bookmark), switcher,
		FXSwitcher::ID_OPEN_SECOND, BUTTON_NORMAL|LAYOUT_FILL_X);

	frame=new FXVerticalFrame(switcher, FRAME_NORMAL|LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
	_playList=new playlist(frame, this, mainWindow::ID_PLAYLIST);
	_playList->setNumVisible(10);

	_bookmarkList=new bookmark(switcher, this, mainWindow::ID_BOOKMARKS);
	_bookmarkList->dropEnable();

	// Right view - dir list
	frame=new FXVerticalFrame(_mainView, FRAME_NORMAL|LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
	/*buttons=new FXHorizontalFrame(frame, LAYOUT_FILL_X|PACK_UNIFORM_WIDTH, 0,0,0,0, 0,0,0,0, 0,0);
	switcher=new FXSwitcher(frame, LAYOUT_FILL|FRAME_NONE, 0,0,0,0, 0,0,0,0);*/

	new FXButton(frame, tr("&Dirlist"), GET_ICON(a, icon_viewtreelist), 0,
		0, BUTTON_NORMAL|LAYOUT_FILL_X);
	/*new FXButton(buttons, tr(""), 0, switcher,
		FXSwitcher::ID_OPEN_SECOND, BUTTON_NORMAL|LAYOUT_FILL_X);*/

	_dirList=new FXDirList(frame, this, mainWindow::ID_DIRLIST,
		TREELIST_SHOWS_BOXES|DIRLIST_SHOWFILES|TREELIST_SINGLESELECT|LAYOUT_FILL);
	_dirList->dropDisable();
	_dirList->setPattern(plExt());
	_dirList->setMatchMode(FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD);

	//_bookmarkList=new bookmark(switcher, this, mainWindow::ID_BOOKMARKS);

	_inputCfg=new FXMenuCommand(_cfgMenu, FXString::null, 0, this, mainWindow::ID_INPUT_CONFIG);

	for (int i=0; i<IFACTORY->num(); i++)
		_fileTypes->appendItem(IFACTORY->name(i));

	_fileTypes->setNumVisible((_fileTypes->getNumItems()>5)?5:_fileTypes->getNumItems());

	// Create a lock file to allow only one instance
	if (FXPipe::create(_lockFileName, FXIO::OwnerReadWrite)) {
		_lockFile=new FXFile(_lockFileName, FXIO::ReadWrite|FXIO::NonBlocking);
		if (_lockFile)
			getApp()->addInput(_lockFile->handle(), INPUT_READ, this, mainWindow::ID_MESSAGE);
	}

	_width=getApp()->reg().readIntEntry("General Settings", "width");
	if (!_width)
		_width=getWidth();
	else
		setWidth(_width);

	_height=getApp()->reg().readIntEntry("General Settings", "height");
	if (!_height)
		_height=getHeight();
	else
		setHeight(_height);
}

mainWindow::~mainWindow() {

	delete _cfgMenu;

	delete ledRedIcon;
	delete ledYellowIcon;
	delete ledGreenIcon;
}

void mainWindow::create() {

	FXMainWindow::create();

	ledGreenIcon->create();
	ledYellowIcon->create();

	// Load settings
	FXString tmp=getApp()->reg().readStringEntry("General Settings", "sort", "ascending");
	_playList->sorting(tmp.text());

	SETTINGS->load(getApp());
	setOpts();

	const FXbool playMode=getApp()->reg().readBoolEntry("General Settings", "play mode", TRUE);
	 _playMode->setState(playMode);

	// Look at command line arguments
	if (getApp()->getArgc()>1)
		setFile(getApp()->getArgv()[1], true);
	else
		setFile(getApp()->reg().readStringEntry("General Settings", "file"), false);

	show(PLACEMENT_SCREEN);
}

void mainWindow::setFile(const FXString& file, bool play) {

	FXTRACE((10, "setFile %s\n", file.text()));

	const FXString fName=FXPath::name(file);

	if (_input &&
	FXPath::match(_input->ext(), fName, FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD))
		goto ok;

	for (int i=0; i<IFACTORY->num(); i++) {
		if (IFACTORY->supports(i, fName.text())) {
			setBackend(i);
			goto ok;
		}
	}

	setBackend(0);

ok:
	if (FXStat::isDirectory(file) || plExt().find(FXPath::extension(file).lower())>=0) {
		_dirList->setCurrentFile(file, TRUE);
		 _playMode->setState(TRUE);
	} else {
		_dirList->setCurrentFile(FXPath::directory(file), TRUE);
		const FXint val=_playList->findItem(file);
		if (val>=0) {
			fxmessage("Loading song, please wait.\n");
			_playList->killSelection();
			_playList->setCurrentItem(val, TRUE);
			_curSong=file;
			if (play)
				_playMode->setState(FALSE);
		}
	}
	_playing=play;
}

FXbool mainWindow::close(FXbool notify) {

	// Save Settings
	getApp()->reg().writeIntEntry("General Settings", "width", _mainView->shown()?getWidth():_width);
	getApp()->reg().writeIntEntry("General Settings", "height", _mainView->shown()?getHeight():_height);

	getApp()->reg().writeStringEntry("General Settings", "file", _curSong.text());
	getApp()->reg().writeBoolEntry("General Settings", "play mode", _playMode->getState());

	getApp()->reg().writeStringEntry("General Settings", "sort", _playList->sorting());

	_bookmarkList->save();

	delete _audio;
	delete _input;

	if (_lockFile) {
		_lockFile->close();
		delete _lockFile;
		FXFile::remove(_lockFileName);
	}

	return FXMainWindow::close(notify);
}

void mainWindow::setOpts() {

	_playList->setAltBackColor(SETTINGS->altCol());
	_bookmarkList->setAltBackColor(SETTINGS->altCol());
}

long mainWindow::onMessage(FXObject*, FXSelector, void*) {

	FXTRACE((10, "onMessage\n"));

	FXchar buffer[2048];
	const int n=_lockFile->readBlock(buffer, 2048);

	const FXString tmp=FXString(buffer, n);

	if (!tmp.empty())
		setFile(tmp, true);

	return 1;
}

long mainWindow::onCmdInfo(FXObject*, FXSelector sel, void*) {

	FXDialogBox* dialog=0;
	switch(FXSELID(sel)) {
	case ID_INFO:
		dialog=new infoDialog(this, _input, FXPath::directory(_curSong).text());
		break;
	case ID_ABOUT:
		dialog=new aboutDialog(this, GET_ICON(getApp(), icon_fap32));
		break;
	}

	dialog->execute(PLACEMENT_OWNER);
	delete dialog;
	return 1;
}

long mainWindow::onCmdConfig(FXObject*, FXSelector sel, void*) {

	switch(FXSELID(sel)) {
	case ID_AUDIO_CONFIG:
		_audio->config(this);
		break;
	case ID_OPT_CONFIG:
		settingsWindow cWin(this);
		if (cWin.execute(PLACEMENT_OWNER)) {
			SETTINGS->save(getApp());
			setOpts();
		} else {
			SETTINGS->load(getApp());
		}
		break;
	}
	return 1;
}

long mainWindow::onCmdInputConfig(FXObject*, FXSelector, void*) {

	if (_playing)
		handle(this, FXSEL(SEL_COMMAND, ID_STOP), 0);
	if (_input->config(this) && _playList->getNumItems()) {
		// if settings changes we must reload the song
		const FXint curItem=_playList->getCurrentItem();
		_playList->killSelection();
		_playList->selectItem(curItem, TRUE);
	}
	return 1;
}

long mainWindow::onCmdDirList(FXObject*, FXSelector, void* autoBk) {

	if ((!_input) || _editMode->getState())
		return 0;

	const FXString curItem=_dirList->getCurrentFile();
	if (curItem.empty())
		return 0;
/*
	if (FXPath::match(_input->ext(), curItem, FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD))
		return 0;
*/
	FXTRACE((10, "Set dir %s\n", curItem.text()));

	_playList->clearItems();

	FXString* files;
	FXint items=(_dirList->isItemFile(_dirList->getCurrentItem()))?
		plLoad(files, curItem):
		FXDir::listFiles(files, curItem, "*", FXDir::NoDirs);

	if (!items) {
		if (!_playing)
			_curSong=FXString::null;
		return 1;
	}

	int fItems=_input->filter(files, items);
	if (!fItems) {
		if (_playing || !SETTINGS->autoBk() || !autoBk)
			goto finish;
		int bk=0;
		input* i=0;
		do {
			if (bk!=_fileTypes->getCurrentItem()) {
				i=backend(bk);
				fItems=i->filter(files, items);
				delete i;
			}
		} while (++bk<_fileTypes->getNumItems() && !fItems);
		if (fItems)
			setBackend(bk-1, 0);
		else
			goto finish;
	}

	items=fItems;

	{
	const FXString path=(FXStat::isDirectory(curItem)?curItem:FXPath::directory(curItem))+PATHSEP;
	while (items--) {
		FXString file=files[items];
		if (!FXPath::isAbsolute(file))
			file.prepend(path);
		FXStat data;
		FXStat::statFile(file, data);
		if (data.isFile() && data.isReadable())
			_playList->appendItem(file);
		FXTRACE((10, "%s\n", file.text()));
	}
	}

	if (!_playList->getNumItems())
		goto finish;

	_playList->sortItems();
	_playList->killSelection();

	if (!_playing) {
		_playList->setCurrentItem(0, TRUE);
		_curSong=_playList->getItemText(0);
	} else if (!compare(_playDir, curItem)) {
		FXint i=_playList->findItem(_curSong);
		_playList->setCurrentItem(i);
		_playList->makeItemVisible(i);
	}

finish:
	delete [] files;
	return 1;
}

long mainWindow::onRgtClkDirList(FXObject*, FXSelector, void* ptr) {

	FXEvent* event=(FXEvent*)ptr;
	if (event->moved)
		return 1;

	FXTreeItem* item=_dirList->getItemAt(event->win_x, event->win_y);
	if (!item)
		return 0;

	FXMenuPane pane(this);
	new FXLabel(&pane, shrink(item->getText()));
	new FXMenuSeparator(&pane);

	if ((_editMode->getState()) && (_dirList->isItemFile(item)))
		new FXMenuCommand(&pane, tr("&Add to playlist\t\tAdd song to the playlist."),
			GET_ICON(getApp(), icon_listadd), this, mainWindow::ID_PL_ADD);
	else
		new FXMenuCommand(&pane, tr("&Add to bookmarks\t\tAdd directory to the bookmarks."),
			GET_ICON(getApp(), icon_bookmark), this, mainWindow::ID_BM_ADD);

	new FXMenuCommand(&pane, tr("Cancel\t\tCancel action"), GET_ICON(getApp(), icon_actioncancel));

	pane.create();
	setUserData((void*)item);
	pane.popup(0, event->root_x, event->root_y);
	getApp()->runModalWhileShown(&pane);
	return 1;
}

long mainWindow::onCmdPlaySong(FXObject*, FXSelector, void*) {

	if (_audio->play(_input)) {
		_playing=true;
		_playDir=_dirList->getCurrentFile();
#ifdef _WIN32
		if (SETTINGS->nowPlaying())
			nowPlaying(true, SETTINGS->wlm(), _input->getInfo(TITLE), _input->getInfo(ARTIST), _input->getInfo(ALBUM));
#endif
	}
	return 1;
}

long mainWindow::onCmdStopSong(FXObject*, FXSelector, void*) {

	if (_audio->stop()) {
		_playing=false;
		_playDir=FXString::null;
		_timeDisplay->setText("00:00");
#ifdef _WIN32
		if (SETTINGS->nowPlaying())
			nowPlaying(false, SETTINGS->wlm());
#endif
	}
	return 1;
}

long mainWindow::onCmdPauseSong(FXObject*, FXSelector, void*) {

	_audio->pause();
	return 1;
}

long mainWindow::onCmdChangeSong(FXObject*, FXSelector sel, void*) {

	if (_playing && compare(_playDir, _dirList->getCurrentFile()))
		return 0;

	FXint idx=_playList->findItem(_curSong);

	switch (FXSELID(sel)) {
	case ID_PREV:
		idx--;
		FXTRACE((10, "Previous\n"));
		break;
	case ID_NEXT:
		idx++;
		FXTRACE((10, "Next\n"));
		break;
	}

	if ((idx>=0) && (idx<_playList->getNumItems())) {
		if (!_preloaded.empty()) {
			_preload->close();
			_preloaded=FXString::null;
		}
		_playList->setCurrentItem(idx, TRUE);
	}

	return 1;
}

long mainWindow::onCmdSongSelected(FXObject*, FXSelector, void* ptr) {

	const FXint curItem=(FXint)(FXival)ptr;
	_playList->makeItemVisible(curItem);
	_curSong=_playList->getItemText(curItem);

	if (!_preloaded.empty()) {
		if(compare(_preloaded, _curSong)) {
			_preload->close();
			_preloaded=FXString::null;
		} else {
			input *temp=_input;
			_input=_preload;
			_preload=temp;
			_preload->close();
			_preloaded=FXString::null;
			setDisplay();
#ifdef _WIN32
			if (SETTINGS->nowPlaying())
				nowPlaying(true, SETTINGS->wlm(), _input->getInfo(TITLE), _input->getInfo(ARTIST), _input->getInfo(ALBUM));
#endif
			return 1;
		}
	}

	if (_loadThread) {
		_loadThread->join();
		delete _loadThread;
	} else {
		getApp()->beginWaitCursor();
		_audio->stop();
		clearDisplay(true);
	}

	load(_curSong, _input);

	return 1;
}

long mainWindow::onCmdSongLoaded(FXObject*, FXSelector, void* ptr) {

	_loadThread->join();

	if (!_preloaded.empty()) {
		delPtr(_loadThread);
		if (ptr && _audio->gapless(_preload)) {
			if (SETTINGS->subtunes())
				_preload->subtune(1);
		} else {
			_preloaded=FXString::null;
		}
		return 1;
	}

	if ((FXint)(FXival)ptr) {
		setDisplay();

		if (SETTINGS->subtunes())
			_input->subtune(1);

		if (_playing) {
#ifdef _WIN32
			if (SETTINGS->nowPlaying())
				nowPlaying(true, SETTINGS->wlm(), _input->getInfo(TITLE), _input->getInfo(ARTIST), _input->getInfo(ALBUM));
#endif
			handle(this, FXSEL(SEL_COMMAND, ID_PLAY), 0);
		}
		FXTRACE((10, "Song loaded\n"));
	} else {
		_playing=false;
		_songInfo->setText(FXString::null);
		fxmessage("Error loading song\n");
	}

	getApp()->endWaitCursor();
	delPtr(_loadThread);
	return 1;
}

void mainWindow::setDisplay() {

	_timeDisplay->setText("00:00");
	_songTime->setText(FXStringFormat("%.2d:%.2d", _input->time()/60, _input->time()%60));

	FXString songTitle=_input->getInfo(TITLE);
	if (songTitle.empty())
		songTitle=FXPath::title(_curSong);

	setTitle(songTitle);

	if (_input->getInfo(ARTIST))
		songTitle=FXStringFormat("%s [%s]", _input->getInfo(TITLE), _input->getInfo(ARTIST));

	_songInfo->setText(songTitle);
	_songInfo->setTipText(songTitle);
}

void mainWindow::clearDisplay(bool const loading) {

	_songInfo->setText(loading?tr("Loading..."):FXString::null);
	_songInfo->setTipText(FXString::null);
	setTitle(FXString(PACKAGE_STRING));
	_songTime->setText("--:--");
	_timeDisplay->setText("--:--");
}

long mainWindow::onCmdAddItem(FXObject*, FXSelector sel, void*) {

	const FXTreeItem* item=(FXTreeItem*)getUserData();
	const FXString itemName=_dirList->getItemPathname(item);
	switch(FXSELID(sel)) {
	case ID_BM_ADD:
		_bookmarkList->add(itemName);
		break;
	case ID_PL_ADD:
		_playList->add(itemName);
		break;
	}
	return 1;
}

long mainWindow::onCmdBmGet(FXObject*, FXSelector, void* data) {

	_dirList->setCurrentFile(*(FXString*)data, TRUE);
	_dirList->expandTree(_dirList->getCurrentItem());
	return 1;
}

void mainWindow::load(FXString filename, input* const i) {

	FXTRACE((10, "Loading %s\n", filename.text()));

	FXGUISignal* songLoaded=new FXGUISignal(getApp(), this, mainWindow::ID_SONG_LOADED);
	_loadThread=new loadThread(i, filename, songLoaded);

	_loadThread->start();
}

long mainWindow::onAudioMsg(FXObject*, FXSelector, void* data) {

	switch ((int)data) {
	case PRELOAD_SONG:
		if (_playMode->getState() && (!compare(_playList->sorting(), "ascending")) &&
		_input->gapless() && !compare(_playDir, _dirList->getCurrentFile())) {
			const FXint nextSong=_playList->getCurrentItem()+1;
			if (nextSong<_playList->getNumItems()) {
				_preloaded=_playList->getItemText(nextSong);
				load(_preloaded, _preload);
			}
		}
	case UPD_TIME:
		_timeDisplay->setText(FXStringFormat("%.2d:%.2d", _audio->seconds()/60, _audio->seconds()%60));
		return 1;
	case SONG_ENDED:
		if (SETTINGS->subtunes() && (_input->subtune()<_input->subtunes())) {
			handle(this, FXSEL(SEL_COMMAND, ID_NEXT_SUBTUNE), 0);
			return 1;
		}

		if (_playMode->getState()) {
			FXint item=0;
			if (!compare(_playDir, _dirList->getCurrentFile()))
				item=_playList->findItem(_curSong)+1;

			if (item<_playList->getNumItems()) {
				_playList->setCurrentItem(item, TRUE);
				return 1;
			}
		}
		handle(this, FXSEL(SEL_COMMAND, ID_STOP), 0);

		return 1;
	}

	return 0;
}

long mainWindow::onCmdUIMode(FXObject* obj, FXSelector, void* ptr) {

	_mainView->handle(obj, FXSEL(SEL_COMMAND, ID_TOGGLESHOWN), ptr);
	if (_mainView->shown()) {
		setDecorations(DECOR_ALL);
		resize(_width, _height);
		_statusbar->setCornerStyle(TRUE);
	} else {
		_width=getWidth();
		_height=getHeight();
		setDecorations(DECOR_TITLE|DECOR_BORDER|DECOR_MINIMIZE|DECOR_CLOSE);
		resize(_toolbar->getDefaultWidth(), getDefaultHeight());
		_statusbar->setCornerStyle(FALSE);
	}

	return 1;
}

long mainWindow::onCmdFiletype(FXObject*, FXSelector, void* ptr) {

	setBackend((FXint)(FXival)ptr, 2);
	return 1;
}

input* mainWindow::backend(int val) {
	return IFACTORY->get(getApp(), val);
}

void mainWindow::setBackend(int val, int refresh) {

	FXTRACE((10, "setBackend %d\n", val));
	if (val<0 || val>_fileTypes->getNumItems())
		return;

	_fileTypes->setCurrentItem(val);

	if (_input) {
		_bookmarkList->save();
		clearDisplay(false);
		delete _input;
		delete _preload;
	}

	_input=backend(val);
	_preload=(_input->gapless())?backend(val):0;

	_inputCfg->setIcon(_input->icon());
	FXString bkName=FXString(IFACTORY->name(val));

	_inputCfg->setText(bkName);
	bkName.append(" config");
	_inputCfg->setHelpText(bkName);

	_bookmarkList->load(IFACTORY->name(val));

	if (refresh)
		handle(this, FXSEL(SEL_CHANGED, ID_DIRLIST), (void*)(FXival)((refresh==2)?0:1));
}

long mainWindow::onUpdFiletype(FXObject* sender, FXSelector, void*) {

	return sender->handle(this,
		((_audio->state()==STOP) && (!_loadThread) && (!_editMode->getState()))?
			FXSEL(SEL_COMMAND, ID_ENABLE):FXSEL(SEL_COMMAND, ID_DISABLE),
		0);
}

long mainWindow::onUpdLed(FXObject* sender, FXSelector, void*) {

	FXLabel* led=(FXLabel*)sender;
	switch (_audio->state()) {
	case STOP:
		led->setIcon(ledRedIcon);
		led->setHelpText(tr("Stopped"));
		break;
	case PLAY:
		led->setIcon(ledGreenIcon);
		led->setHelpText(tr("Playing"));
		break;
	case PAUSE:
		led->setIcon(ledYellowIcon);
		led->setHelpText(tr("Paused"));
		break;
	}
	return 1;
}

long mainWindow::onUpdSubtune(FXObject* sender, FXSelector sel, void*) {

	int tune, tunes;
	switch (FXSELID(sel)) {
	case ID_PLSONG:
		if (_playing && compare(_playDir, _dirList->getCurrentFile()))
			return 0;
		tunes=_playList->getNumItems();
		tune=1+_playList->getCurrentItem();
		break;
	case ID_SUBTUNE:
		tune=_input->subtune();
		tunes=_input->subtunes();
		break;
	}

	FXLabel* label=(FXLabel*)sender;
	label->setText(FXStringFormat("%.2d/%.2d", tune, tunes));
	return 1;
}

long mainWindow::onCmdSubtune(FXObject*, FXSelector sel, void*) {

	if (_input->subtunes()<=1)
		return 0;

	FXuint i=_input->subtune();
	switch (FXSELID(sel)) {
	case ID_PREV_SUBTUNE:
		i--;
		break;
	case ID_NEXT_SUBTUNE:
		i++;
		break;
	}

	if ((i<1) || (i>_input->subtunes()))
		return 0;

	_audio->stop();

	if (_input->subtune(i))
		setDisplay();

	if (_playing)
		handle(this, FXSEL(SEL_COMMAND, ID_PLAY), 0);
	return 1;
}

long mainWindow::onUpdInfoDisplay(FXObject*, FXSelector, void*) {

	_songInfo->makePositionVisible(0);
	return 1;
}

long mainWindow::onCmdPlEdit(FXObject*, FXSelector, void* ptr) {

	if (ptr) {
		_playList->clearItems();
		_playList->dropEnable();
		_dirList->setPattern(_input->ext());
	} else {
		_playList->dropDisable();
		_dirList->setPattern(plExt());
		handle(this, FXSEL(SEL_CHANGED, ID_DIRLIST), 0);
	}

	return 1;
}

long mainWindow::onCmdPlSave(FXObject* sender, FXSelector, void*) {

	const FXint n=_playList->getNumItems();
	if (!n)
		return 0;

	FXString filename=FXFileDialog::getSaveFilename(this, tr("Save playlist"), "", plExt());
	if (filename.empty())
		return 0;

	if (FXStat::exists(filename))
		if (FXMessageBox::question(this, MBOX_YES_NO, tr("Warning"), tr("File exists, owerwrite?"))==MBOX_CLICKED_NO)
			return 0;

	FXString* files=new FXString [n];
	const FXString path=(_editMode->getState())?FXString::null:_dirList->getCurrentFile()+PATHSEP;
	for (int i=0; i<n; i++)
		files[i]=path+_playList->getItemText(i);

	if (!plSave(files, n, filename))
		FXMessageBox::error(this, MBOX_OK, tr("Error"), tr("Error saving playlist"));

	delete [] files;

	return 1;
}

long mainWindow::onCmdVol(FXObject*, FXSelector, void* data) {

	_audio->volume((FXint)(FXival)data);
	return 1;
}
