/*
 *  Copyright (C) 2008-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "settings.h"
#include "iconFactory.h"

FXDEFMAP(settingsWindow) settingsWindowMap[] = {
	// Message_Type		ID			Message_Handler
	FXMAPFUNC(SEL_COMMAND,	settingsWindow::ID_COLOR,	settingsWindow::onCmdColor),
};

FXIMPLEMENT(settingsWindow, FXDialogBox, settingsWindowMap, ARRAYNUMBER(settingsWindowMap))

settingsWindow::settingsWindow(FXWindow* win) :
	FXDialogBox(win, FXString::null, DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0) {

	setTitle(tr("Settings"));

	_subtunes.connect(settings::instance()->_subtunes);
	_autoBk.connect(settings::instance()->_autoBk);
	_albumArt.connect(settings::instance()->_albumArt);
	_aaSize.connect(settings::instance()->_aaSize);
#ifdef _WIN32
	_nowPlaying.connect(settings::instance()->_nowPlaying);
	_wlm.connect(settings::instance()->_wlm);
#endif
#ifdef HAVE_BS2B
	_bs2b.connect(settings::instance()->_bs2b);
#endif

	FXVerticalFrame* main=new FXVerticalFrame(this, LAYOUT_FILL_X);

	FXGroupBox* gb=new FXGroupBox(main, tr("Options"), GROUPBOX_NORMAL|FRAME_GROOVE|LAYOUT_FILL_X);
	new FXCheckButton(gb, tr("&Play subtunes\tPlay all subtunes"), &_subtunes, FXDataTarget::ID_VALUE);
	new FXCheckButton(gb, tr("&Auto backend\tAutomatic backend selection"), &_autoBk, FXDataTarget::ID_VALUE);
#ifdef HAVE_BS2B
	new FXCheckButton(gb, tr("Bauer stereophonic-to-binaural DSP"), &_bs2b, FXDataTarget::ID_VALUE);
#endif

	new FXCheckButton(gb, tr("Album art\tShow album art"), &_albumArt, FXDataTarget::ID_VALUE);
	FXHorizontalFrame* hf=new FXHorizontalFrame(gb, LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
	new FXLabel(hf, tr("Album art max size\tMaximum size for album art"));
	FXSpinner* spin=new FXSpinner(hf, 3, &_aaSize, FXDataTarget::ID_VALUE, FRAME_SUNKEN|TEXTFIELD_NORMAL|TEXTFIELD_INTEGER);
	spin->setRange(0, 500);
	spin->setIncrement(10);

	gb=new FXGroupBox(main, tr("Colors"), GROUPBOX_NORMAL|FRAME_GROOVE|LAYOUT_FILL_X);
	hf=new FXHorizontalFrame(gb, LAYOUT_FILL, 0,0,0,0, 0,0,0,0, 0,0);
	new FXLabel(hf, tr("Alt. color\tAlternate color for lists"));
	new FXColorWell(hf, settings::instance()->_altCol, this, settingsWindow::ID_COLOR,
		COLORWELL_OPAQUEONLY|COLORWELL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL,
		0, 0, 40, 24);

#ifdef _WIN32
	gb=new FXGroupBox(main, tr("MSN Messenger Now Playing"), GROUPBOX_NORMAL|FRAME_GROOVE|LAYOUT_FILL_X);
	new FXCheckButton(gb, tr("Enable"), &_nowPlaying, FXDataTarget::ID_VALUE);
	new FXCheckButton(gb, tr("Live Messenger"), &_wlm, FXDataTarget::ID_VALUE);
#endif

	new FXSeparator(this);

	FXHorizontalFrame* bottom=new FXHorizontalFrame(this, LAYOUT_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
	FXButton* initial=new FXButton(bottom, tr("&OK"), GET_ICON(getApp(), icon_dialogok),
		this, ID_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X);
	new FXButton(bottom, tr("&Cancel"), GET_ICON(getApp(), icon_dialogcancel),
		this, ID_CANCEL, BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X);
	initial->setFocus();
}

long settingsWindow::onCmdColor(FXObject*, FXSelector, void* ptr) {

	settings::instance()->_altCol=(FXColor)(FXuval)ptr;
	return 1;
}

/*****************************************************************/

settings* settings::instance() {

	static settings s;
	return &s;
}

void settings::load(FXApp* a) {

	_subtunes=a->reg().readBoolEntry("General Settings", "play subtunes", FALSE);
	_autoBk=a->reg().readBoolEntry("General Settings", "auto backend", TRUE);
	_altCol=a->reg().readColorEntry("General Settings", "alternate color", -1);
#ifdef HAVE_BS2B
	_bs2b=a->reg().readBoolEntry("General Settings", "Bauer DSP", FALSE);
#endif
	_albumArt=a->reg().readBoolEntry("General Settings", "album art", TRUE);
	_aaSize=a->reg().readIntEntry("General Settings", "album art max size", 300);
#ifdef _WIN32
	_nowPlaying=a->reg().readBoolEntry("General Settings", "messenger now playing", FALSE);
	_wlm=a->reg().readBoolEntry("General Settings", "live messenger", TRUE);
#endif
}

void settings::save(FXApp* a) {

	a->reg().writeBoolEntry("General Settings", "play subtunes", _subtunes);
	a->reg().writeBoolEntry("General Settings", "auto backend", _autoBk);
	a->reg().writeColorEntry("General Settings", "alternate color", _altCol);
#ifdef HAVE_BS2B
	a->reg().writeBoolEntry("General Settings", "Bauer DSP", _bs2b);
#endif
	a->reg().writeBoolEntry("General Settings", "album art", _albumArt);
	a->reg().writeIntEntry("General Settings", "album art max size", _aaSize);
#ifdef _WIN32
	a->reg().writeBoolEntry("General Settings", "messenger now playing", _nowPlaying);
	a->reg().writeBoolEntry("General Settings", "live messenger", _wlm);
#endif
}
