/*
 *  Copyright (C) 2008-2009 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef SETTINGS_H
#define SETTINGS_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <fx.h>

class settingsWindow : public FXDialogBox {
	FXDECLARE(settingsWindow)

private:
	FXDataTarget	_subtunes;
	FXDataTarget	_autoBk;
	FXDataTarget	_albumArt;
	FXDataTarget	_aaSize;
#ifdef _WIN32
	FXDataTarget	_nowPlaying;
	FXDataTarget	_wlm;
#endif
#ifdef HAVE_BS2B
	FXDataTarget	_bs2b;
#endif

private:
	settingsWindow() {};
	settingsWindow(const settingsWindow&);
	settingsWindow& operator=(const settingsWindow&);

public:
	settingsWindow(FXWindow* win);
	virtual ~settingsWindow() {};

public:
	enum{
		ID_COLOR=FXDialogBox::ID_LAST,
		ID_LAST,
	};

public:
	long onCmdColor(FXObject*, FXSelector, void*);
};

/*****************************************************************/

#define SETTINGS	settings::instance()

class settings {

	friend class settingsWindow;

private:
	FXbool	_subtunes;
	FXbool	_autoBk;
	FXColor	_altCol;
	FXbool	_albumArt;
	FXint	_aaSize;
#ifdef _WIN32
	FXbool	_nowPlaying;
	FXbool	_wlm;
#endif
#ifdef HAVE_BS2B
	FXbool	_bs2b;
#endif

protected:
	settings() {};
	settings(const settings&);
	settings& operator= (const settings&);
	~settings() {};

public:
	/// Get singloton instance
	static settings* instance();

	/// Load setting
	void load(FXApp* a);

	/// Save settings
	void save(FXApp* a);

	// Get program settings

	/// Play subtunes
	FXbool subtunes() { return _subtunes; }

	/// Automatic backend selection
	FXbool autoBk() { return _autoBk; }

	/// Alternate color for lists
	FXColor altCol() { return _altCol; }

	/// Show album art
	FXbool albumArt() { return _albumArt; }

	/// Album art max size
	FXint aaSize() { return _aaSize; }

#ifdef _WIN32
	/// Windows Messenger Now Playing
	FXbool nowPlaying() { return _nowPlaying; }

	/// Windows Live Messanger
	FXbool wlm() { return _wlm; }
#endif
#ifdef HAVE_BS2B
	/// Bauer stereophonic-to-binaural DSP
	FXbool bs2b() { return _bs2b; }
#endif
};

#endif
