/*
 *  Copyright (C) 2006-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <fx.h>
#include <xincs.h>

#include "gui/mainWindow.h"

int main(int argc, char *argv[]) {

#ifdef DEBUG
fxTraceLevel=20;
#endif

	FXString lockFileName=FXStringFormat("%s%cfap.lock", FXSystem::getTempDirectory().text(), PATHSEP);

	if (FXStat::exists(lockFileName)) {
		// Another instance is running, pass cmd line args if any
		if (argc>1) {
			FXFile file(lockFileName, FXIO::WriteOnly|FXIO::NonBlocking);
			FXString arg=argv[1];
			file.writeBlock(arg.text(), arg.length());
			file.close();
		}
		fxmessage("Already running\n");
		return -1;
	}

	// Make application
	FXApp application(PACKAGE_NAME, "DrFiemost");

	// Start app
	application.init(argc, argv);

	// Make a tool tip
	new FXToolTip(&application, TOOLTIP_VARIABLE);

	mainWindow* sWin=new mainWindow(&application, lockFileName.text());

	// Catch interrupt so we can gracefully terminate
	application.addSignal(SIGINT, sWin, mainWindow::ID_CLOSE);

	// Create the application's windows
	application.create();

	return application.run();
}
